@@ -1,55 +1,17 @@
-/* ====================================================================
- * The Apache Software License, Version 1.1
+/* Copyright 2000-2005 The Apache Software Foundation or its licensors, as
+ * applicable.
  *
- * Copyright (c) 2000-2003 The Apache Software Foundation.  All rights
- * reserved.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Apache" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 #include "apr_arch_networkio.h"
@@ -79,8 +41,8 @@ apr_status_t apr_socket_send(apr_socket_t *sock, const char *buf,
         rv = write(sock->socketdes, buf, (*len));
     } while (rv == -1 && errno == EINTR);
 
-    if (rv == -1 && (errno == EAGAIN || errno == EWOULDBLOCK) 
-      && sock->timeout != 0) {
+    while (rv == -1 && (errno == EAGAIN || errno == EWOULDBLOCK) 
+        && apr_is_option_set(sock->netmask, APR_SO_TIMEOUT)) {
         apr_status_t arv;
 do_select:
         arv = apr_wait_for_io_or_timeout(NULL, sock, 0);
@@ -98,8 +60,8 @@ do_select:
         *len = 0;
         return errno;
     }
-    if (sock->timeout && rv < *len) {
-    sock->netmask |= APR_INCOMPLETE_WRITE;
+    if (apr_is_option_set(sock->netmask, APR_SO_TIMEOUT) && rv < *len) {
+        sock->netmask |= APR_INCOMPLETE_WRITE;
     }
     (*len) = rv;
     return APR_SUCCESS;
@@ -111,7 +73,7 @@ apr_status_t apr_socket_recv(apr_socket_t *sock, char *buf, apr_size_t *len)
     apr_status_t arv;
 
     if (sock->netmask & APR_INCOMPLETE_READ) {
-    sock->netmask &= ~APR_INCOMPLETE_READ;
+        sock->netmask &= ~APR_INCOMPLETE_READ;
         goto do_select;
     }
 
@@ -119,10 +81,10 @@ apr_status_t apr_socket_recv(apr_socket_t *sock, char *buf, apr_size_t *len)
         rv = read(sock->socketdes, buf, (*len));
     } while (rv == -1 && errno == EINTR);
 
-    if (rv == -1 && (errno == EAGAIN || errno == EWOULDBLOCK) && 
-      sock->timeout != 0) {
+    while (rv == -1 && (errno == EAGAIN || errno == EWOULDBLOCK) && 
+        apr_is_option_set(sock->netmask, APR_SO_TIMEOUT)) {
 do_select:
-    arv = apr_wait_for_io_or_timeout(NULL, sock, 1);
+        arv = apr_wait_for_io_or_timeout(NULL, sock, 1);
         if (arv != APR_SUCCESS) {
             *len = 0;
             return arv;
@@ -137,8 +99,8 @@ do_select:
         (*len) = 0;
         return errno;
     }
-    if (sock->timeout && rv < *len) {
-    sock->netmask |= APR_INCOMPLETE_READ;
+    if (apr_is_option_set(sock->netmask, APR_SO_TIMEOUT) && rv < *len) {
+        sock->netmask |= APR_INCOMPLETE_READ;
     }
     (*len) = rv;
     if (rv == 0) {
@@ -159,8 +121,8 @@ apr_status_t apr_socket_sendto(apr_socket_t *sock, apr_sockaddr_t *where,
                     where->salen);
     } while (rv == -1 && errno == EINTR);
 
-    if (rv == -1 && (errno == EAGAIN || errno == EWOULDBLOCK)
-        && sock->timeout != 0) {
+    while (rv == -1 && (errno == EAGAIN || errno == EWOULDBLOCK)
+        && apr_is_option_set(sock->netmask, APR_SO_TIMEOUT)) {
         apr_status_t arv = apr_wait_for_io_or_timeout(NULL, sock, 0);
         if (arv != APR_SUCCESS) {
             *len = 0;
@@ -192,8 +154,8 @@ apr_status_t apr_socket_recvfrom(apr_sockaddr_t *from, apr_socket_t *sock,
                       (struct sockaddr*)&from->sa, &from->salen);
     } while (rv == -1 && errno == EINTR);
 
-    if (rv == -1 && (errno == EAGAIN || errno == EWOULDBLOCK) &&
-        sock->timeout != 0) {
+    while (rv == -1 && (errno == EAGAIN || errno == EWOULDBLOCK) &&
+        apr_is_option_set(sock->netmask, APR_SO_TIMEOUT)) {
         apr_status_t arv = apr_wait_for_io_or_timeout(NULL, sock, 1);
         if (arv != APR_SUCCESS) {
             *len = 0;
@@ -202,7 +164,7 @@ apr_status_t apr_socket_recvfrom(apr_sockaddr_t *from, apr_socket_t *sock,
             do {
                 rv = recvfrom(sock->socketdes, buf, (*len), flags,
                               (struct sockaddr*)&from->sa, &from->salen);
-                } while (rv == -1 && errno == EINTR);
+            } while (rv == -1 && errno == EINTR);
         }
     }
     if (rv == -1) {
@@ -211,8 +173,9 @@ apr_status_t apr_socket_recvfrom(apr_sockaddr_t *from, apr_socket_t *sock,
     }
 
     (*len) = rv;
-    if (rv == 0 && sock->type == SOCK_STREAM)
+    if (rv == 0 && sock->type == SOCK_STREAM) {
         return APR_EOF;
+    }
 
     return APR_SUCCESS;
 }
@@ -238,8 +201,8 @@ apr_status_t apr_socket_sendv(apr_socket_t * sock, const struct iovec *vec,
         rv = writev(sock->socketdes, vec, nvec);
     } while (rv == -1 && errno == EINTR);
 
-    if (rv == -1 && (errno == EAGAIN || errno == EWOULDBLOCK) && 
-      sock->timeout != 0) {
+    while (rv == -1 && (errno == EAGAIN || errno == EWOULDBLOCK) && 
+        apr_is_option_set(sock->netmask, APR_SO_TIMEOUT)) {
         apr_status_t arv;
 do_select:
         arv = apr_wait_for_io_or_timeout(NULL, sock, 0);
@@ -257,8 +220,9 @@ do_select:
         *len = 0;
         return errno;
     }
-    if (sock->timeout && rv < requested_len) {
-    sock->netmask |= APR_INCOMPLETE_WRITE;
+    if (apr_is_option_set(sock->netmask, APR_SO_TIMEOUT) &&
+        rv < requested_len) {
+        sock->netmask |= APR_INCOMPLETE_WRITE;
     }
     (*len) = rv;
     return APR_SUCCESS;
@@ -305,7 +269,7 @@ apr_status_t apr_socket_sendfile(apr_socket_t *sock, apr_file_t *file,
         arv = apr_socket_sendv(sock, hdtr->headers, hdtr->numheaders,
                                &hdrbytes);
         if (arv != APR_SUCCESS) {
-        *len = 0;
+            *len = 0;
             return errno;
         }
         nbytes += hdrbytes;
@@ -331,33 +295,32 @@ apr_status_t apr_socket_sendfile(apr_socket_t *sock, apr_file_t *file,
 
     do {
         rv = sendfile(sock->socketdes,    /* socket */
-                  file->filedes,    /* open file descriptor of the file to be sent */
-                  &off,    /* where in the file to start */
-                  *len    /* number of bytes to send */
-            );
+                      file->filedes, /* open file descriptor of the file to be sent */
+                      &off,    /* where in the file to start */
+                      *len);   /* number of bytes to send */
     } while (rv == -1 && errno == EINTR);
 
-    if (rv == -1 && 
+    while (rv == -1 && 
         (errno == EAGAIN || errno == EWOULDBLOCK) && 
-        sock->timeout > 0) {
+        apr_is_option_set(sock->netmask, APR_SO_TIMEOUT)) {
 do_select:
-    arv = apr_wait_for_io_or_timeout(NULL, sock, 0);
-    if (arv != APR_SUCCESS) {
-        *len = 0;
-        return arv;
-    }
+        arv = apr_wait_for_io_or_timeout(NULL, sock, 0);
+        if (arv != APR_SUCCESS) {
+            *len = 0;
+            return arv;
+        }
         else {
             do {
-            rv = sendfile(sock->socketdes,    /* socket */
-                      file->filedes,    /* open file descriptor of the file to be sent */
-                      &off,    /* where in the file to start */
-                      *len);    /* number of bytes to send */
+                rv = sendfile(sock->socketdes,    /* socket */
+                              file->filedes, /* open file descriptor of the file to be sent */
+                              &off,    /* where in the file to start */
+                              *len);    /* number of bytes to send */
             } while (rv == -1 && errno == EINTR);
         }
     }
 
     if (rv == -1) {
-    *len = nbytes;
+        *len = nbytes;
         rv = errno;
         apr_socket_opt_set(sock, APR_TCP_NOPUSH, 0);
         return rv;
@@ -374,7 +337,7 @@ do_select:
              * partial byte count;  this is a non-blocking socket.
              */
 
-            if (sock->timeout) {
+            if (apr_is_option_set(sock->netmask, APR_SO_TIMEOUT)) {
                 sock->netmask |= APR_INCOMPLETE_WRITE;
             }
             return arv;
@@ -396,7 +359,7 @@ do_select:
                                &trbytes);
         nbytes += trbytes;
         if (arv != APR_SUCCESS) {
-        *len = nbytes;
+            *len = nbytes;
             rv = errno;
             apr_socket_opt_set(sock, APR_TCP_NOPUSH, 0);
             return rv;
@@ -444,7 +407,7 @@ static int include_hdrs_in_length(void)
     kernel_version_size = sizeof(kernel_version);
     if (sysctlbyname("kern.osreldate", &kernel_version, 
                      &kernel_version_size, NULL, NULL) == 0 &&
-            kernel_version < KERNEL_WITH_SENDFILE_LENGTH_FIX) {
+        kernel_version < KERNEL_WITH_SENDFILE_LENGTH_FIX) {
         api = OLD;
         return 1;
     }
@@ -522,7 +485,7 @@ apr_status_t apr_socket_sendfile(apr_socket_t * sock, apr_file_t * file,
 
             if (rv == -1) {
                 if (errno == EAGAIN) {
-                    if (sock->timeout) {
+                    if (apr_is_option_set(sock->netmask, APR_SO_TIMEOUT)) {
                         sock->netmask |= APR_INCOMPLETE_WRITE;
                     }
                     /* FreeBSD's sendfile can return -1/EAGAIN even if it
@@ -562,7 +525,7 @@ apr_status_t apr_socket_sendfile(apr_socket_t * sock, apr_file_t * file,
         }
         if (rv == -1 &&
             errno == EAGAIN && 
-            sock->timeout > 0) {
+            apr_is_option_set(sock->netmask, APR_SO_TIMEOUT)) {
             apr_status_t arv = apr_wait_for_io_or_timeout(NULL, sock, 0);
             if (arv != APR_SUCCESS) {
                 *len = 0;
@@ -678,9 +641,9 @@ apr_status_t apr_socket_sendfile(apr_socket_t *sock, apr_file_t *file,
         }
     } while (rc == -1 && errno == EINTR);
 
-    if (rc == -1 && 
+    while (rc == -1 && 
         (errno == EAGAIN || errno == EWOULDBLOCK) && 
-        sock->timeout > 0) {
+        apr_is_option_set(sock->netmask, APR_SO_TIMEOUT)) {
         apr_status_t arv = apr_wait_for_io_or_timeout(NULL, sock, 0);
 
         if (arv != APR_SUCCESS) {
@@ -705,7 +668,7 @@ apr_status_t apr_socket_sendfile(apr_socket_t *sock, apr_file_t *file,
     }
 
     if (rc == -1) {
-    *len = 0;
+        *len = 0;
         return errno;
     }
 
@@ -740,6 +703,14 @@ apr_status_t apr_socket_sendfile(apr_socket_t * sock, apr_file_t * file,
     /* Ignore flags for now. */
     flags = 0;
 
+    /* word to the wise: by default, AIX stores files sent by send_file()
+     * in the network buffer cache...  there are supposedly scenarios
+     * where the most recent copy of the file won't be sent, but I can't
+     * recreate the potential problem, perhaps because of the way we
+     * use send_file()...  if you suspect such a problem, try turning
+     * on the SF_SYNC_CACHE flag
+     */
+
     /* AIX can also send the headers/footers as part of the system call */
     parms.header_length = 0;
     if (hdtr && hdtr->numheaders) {
@@ -795,7 +766,9 @@ apr_status_t apr_socket_sendfile(apr_socket_t * sock, apr_file_t * file,
             parms.trailer_data = tbuf;
         }
     }
-    else parms.trailer_data = NULL;
+    else {
+        parms.trailer_data = NULL;
+    }
 
     /* Whew! Headers and trailers set up. Now for the file data */
 
@@ -816,9 +789,9 @@ apr_status_t apr_socket_sendfile(apr_socket_t * sock, apr_file_t * file,
                        flags);             /* flags */
     } while (rv == -1 && errno == EINTR);
 
-    if (rv == -1 &&
+    while (rv == -1 &&
         (errno == EAGAIN || errno == EWOULDBLOCK) &&
-        sock->timeout > 0) {
+        apr_is_option_set(sock->netmask, APR_SO_TIMEOUT)) {
 do_select:
         arv = apr_wait_for_io_or_timeout(NULL, sock, 0);
         if (arv != APR_SUCCESS) {
@@ -846,7 +819,7 @@ do_select:
         return errno;
     }
 
-    if (sock->timeout &&
+    if (apr_is_option_set(sock->netmask, APR_SO_TIMEOUT) &&
         (parms.bytes_sent < (parms.file_bytes + parms.header_length + parms.trailer_length))) {
         sock->netmask |= APR_INCOMPLETE_WRITE;
     }
@@ -913,8 +886,9 @@ apr_status_t apr_socket_sendfile(apr_socket_t *sock, apr_file_t *file,
 
         curvec++;
     }
-    else
+    else {
         vecs--;
+    }
 
     /* Add the footers */
     for (i = 0; i < hdtr->numtrailers; i++, curvec++) {
@@ -955,17 +929,15 @@ apr_status_t apr_socket_sendfile(apr_socket_t *sock, apr_file_t *file,
         /* socket, vecs, number of vecs, bytes written */
         rv = sendfilev(sock->socketdes, sfv, vecs, &nbytes);
 
-        if (rv == -1 && errno == EAGAIN)
-        {
-            if (nbytes)
-                rv = 0; 
+        if (rv == -1 && errno == EAGAIN) {
+            if (nbytes) {
+                rv = 0;
+            }
             else if (!arv &&
-                     apr_is_option_set(sock->netmask, APR_SO_TIMEOUT) == 1)
-            {
+                     apr_is_option_set(sock->netmask, APR_SO_TIMEOUT) == 1) {
                 apr_status_t t = apr_wait_for_io_or_timeout(NULL, sock, 0);
 
-                if (t != APR_SUCCESS)
-                {
+                if (t != APR_SUCCESS) {
                     *len = 0;
                     return t;
                 }
@@ -976,16 +948,16 @@ apr_status_t apr_socket_sendfile(apr_socket_t *sock, apr_file_t *file,
         }
     } while ((rv == -1 && errno == EINTR) || repeat);
 
-    if (rv == -1)
-    {
+    if (rv == -1) {
         *len = 0;
         return errno;
     }
 
     /* Update how much we sent */
     *len = nbytes;
-    if (sock->timeout && (*len < requested_len)) {
-    sock->netmask |= APR_INCOMPLETE_WRITE;
+    if (apr_is_option_set(sock->netmask, APR_SO_TIMEOUT) &&
+        (*len < requested_len)) {
+        sock->netmask |= APR_INCOMPLETE_WRITE;
     }
     return APR_SUCCESS;
 }
