@@ -2,12 +2,12 @@
 #
 # Copyright (c) 2004-2005, Awarix, Inc.
 # All rights reserved.
-# 
+#
 # Subject to the following obligations and disclaimer of warranty,
 # use and redistribution of this software, in source or object code
 # forms, with or without modifications are expressly permitted by
 # Awarix; provided, however, that:
-# 
+#
 #    (i)  Any and all reproductions of the source or object code
 #         must include the copyright notice above and the following
 #         disclaimer of warranties; and
@@ -15,7 +15,7 @@
 #         Awarix trademarks, including the mark "AWARIX"
 #         on advertising, endorsements, or otherwise except as such
 #         appears in the above copyright notice or in the software.
-# 
+#
 # THIS SOFTWARE IS BEING PROVIDED BY AWARIX "AS IS", AND
 # TO THE MAXIMUM EXTENT PERMITTED BY LAW, AWARIX MAKES NO
 # REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED, REGARDING
@@ -41,16 +41,16 @@
 #   John Belmonte <john@neggie.net> - metadata and usability improvements
 #
 # $HeadURL: http://svn.collab.net/repos/svn/branches/1.3.x/contrib/client-side/svnmerge.sh $
-# $LastChangedDate: 2005-10-05 01:35:09 +0200 (Wed, 05 Oct 2005) $
-# $LastChangedBy: maxb $
-# $LastChangedRevision: 16473 $
+# $LastChangedDate: 2006-03-10 15:31:59 +0100 (Fri, 10 Mar 2006) $
+# $LastChangedBy: malcolm $
+# $LastChangedRevision: 18811 $
 
 # Definitions (would like ':' in property names but can't because of bug 1971)
 NAME="svnmerge"
 SVN_MERGE_SVN="svn"
 SVN_MERGE_PROP="${NAME}-integrated"
-SRCREV=`echo '$Rev: 16473 $' | sed 's/^\$Rev: \([0-9]\{1,\}\).\{0,\}$/\1/g'`
-SRCDATE=`echo '$Date: 2005-10-05 01:35:09 +0200 (Wed, 05 Oct 2005) $' | sed 's/^\$Date: .\{0,\}(\(.\{0,\}\)).\{0,\}$/\1/g'`
+SRCREV=`echo '$Rev: 18811 $' | sed 's/^\$Rev: \([0-9]\{1,\}\).\{0,\}$/\1/g'`
+SRCDATE=`echo '$Date: 2006-03-10 15:31:59 +0100 (Fri, 10 Mar 2006) $' | sed 's/^\$Date: .\{0,\}(\(.\{0,\}\)).\{0,\}$/\1/g'`
 
 # We expect non-localized output
 LC_MESSAGES="C"
