@@ -27,23 +27,26 @@
 #endif // _MSC_VER > 1000
 #include <jni.h>
 struct apr_array_header_t;
+struct svn_error_t;
 class Pool;
 #include "Path.h"
 #include <vector>
 
-class Targets  
+class Targets
 {
 private:
     std::vector<Path> m_targets;
-	jobjectArray m_targetArray;
-
+    jobjectArray m_targetArray;
+    svn_error_t *m_error_occured;
+    bool m_doesNotContainsPath;
 public:
-	Targets(jobjectArray jtargets);
-	Targets(const char *path);
-	void add(const char *path);
-	~Targets();
+    Targets(jobjectArray jtargets);
+    Targets(const char *path);
+    void add(const char *path);
+    ~Targets();
     const apr_array_header_t *array (const Pool & pool);
-
+    svn_error_t *error_occured();
+    void setDoesNotContainsPath();
 };
-
-#endif // !defined(AFX_TARGETS_H__61202731_41A4_43FF_97C4_7E26DC255BF1__INCLUDED_)
+// !defined(AFX_TARGETS_H__61202731_41A4_43FF_97C4_7E26DC255BF1__INCLUDED_)
+#endif
