@@ -22,13 +22,9 @@
 
 /*** Includes. ***/
 
-#include "svn_wc.h"
 #include "svn_client.h"
-#include "svn_string.h"
-#include "svn_path.h"
-#include "svn_delta.h"
 #include "svn_error.h"
-#include "svn_pools.h"
+#include "svn_path.h"
 #include "cl.h"
 
 #include "svn_private_config.h"
@@ -36,22 +32,6 @@
 
 /*** Code. ***/
 
-svn_error_t *
-svn_cl__may_need_force (svn_error_t *err)
-{
-  if (err
-      && (err->apr_err == SVN_ERR_UNVERSIONED_RESOURCE ||
-          err->apr_err == SVN_ERR_CLIENT_MODIFIED))
-    {
-      /* Should this svn_error_compose a new error number? Probably not,
-         the error hasn't changed. */
-      err = svn_error_quick_wrap
-        (err, _("Use --force to override this restriction") );
-    }
-
-  return err;
-}
-
 /* This implements the `svn_opt_subcommand_t' interface. */
 svn_error_t *
 svn_cl__delete (apr_getopt_t *os,
@@ -61,28 +41,43 @@ svn_cl__delete (apr_getopt_t *os,
   svn_cl__opt_state_t *opt_state = ((svn_cl__cmd_baton_t *) baton)->opt_state;
   svn_client_ctx_t *ctx = ((svn_cl__cmd_baton_t *) baton)->ctx;
   apr_array_header_t *targets;
-  svn_client_commit_info_t *commit_info = NULL;
+  svn_commit_info_t *commit_info = NULL;
   svn_error_t *err;
 
-  SVN_ERR (svn_opt_args_to_target_array (&targets, os, 
-                                         opt_state->targets,
-                                         &(opt_state->start_revision),
-                                         &(opt_state->end_revision),
-                                         FALSE, pool));
+  SVN_ERR (svn_opt_args_to_target_array2 (&targets, os, 
+                                          opt_state->targets, pool));
 
   if (! targets->nelts)
     return svn_error_create (SVN_ERR_CL_ARG_PARSING_ERROR, 0, NULL);
 
   if (! opt_state->quiet)
-    svn_cl__get_notifier (&ctx->notify_func, &ctx->notify_baton, FALSE, FALSE,
-                          FALSE, pool);
+    svn_cl__get_notifier (&ctx->notify_func2, &ctx->notify_baton2, FALSE,
+                          FALSE, FALSE, pool);
 
-  SVN_ERR (svn_cl__make_log_msg_baton (&(ctx->log_msg_baton), opt_state, 
-                                       NULL, ctx->config, pool));
-  err = svn_client_delete (&commit_info, targets, opt_state->force, ctx, pool);
+  if (! svn_path_is_url (APR_ARRAY_IDX (targets, 0, const char *)))
+    {
+      ctx->log_msg_func2 = NULL;
+      if (opt_state->message || opt_state->filedata)
+        {
+          return svn_error_create
+            (SVN_ERR_CL_UNNECESSARY_LOG_MESSAGE, NULL,
+             _("Local, non-commit operations do not take a log message"));
+        }
+    }
+  else
+    {
+      SVN_ERR (svn_cl__make_log_msg_baton (&(ctx->log_msg_baton2), opt_state,
+                                           NULL, ctx->config, pool));
+    }
+
+  err = svn_client_delete2 (&commit_info, targets, opt_state->force, ctx, pool);
   if (err)
     err = svn_cl__may_need_force (err);
-  SVN_ERR (svn_cl__cleanup_log_msg (ctx->log_msg_baton, err));
+
+  if (ctx->log_msg_func2)
+    SVN_ERR (svn_cl__cleanup_log_msg (ctx->log_msg_baton2, err));
+  else if (err)
+    return err;
 
   if (commit_info && ! opt_state->quiet)
     SVN_ERR (svn_cl__print_commit_info (commit_info, pool));
