@@ -28,12 +28,13 @@
 #include "svn_client.h"
 #include "svn_string.h"
 #include "svn_path.h"
-#include "svn_delta.h"
 #include "svn_error.h"
 #include "svn_utf.h"
-#include "svn_subst.h"
+#include "svn_props.h"
 #include "cl.h"
 
+#include "svn_private_config.h"
+
 
 /*** Code. ***/
 
@@ -56,41 +57,22 @@ svn_cl__propedit (apr_getopt_t *os,
   SVN_ERR (svn_utf_cstring_to_utf8 (&pname_utf8, pname, pool));
 
   /* Suck up all the remaining arguments into a targets array */
-  SVN_ERR (svn_opt_args_to_target_array (&targets, os, 
-                                         opt_state->targets,
-                                         &(opt_state->start_revision),
-                                         &(opt_state->end_revision),
-                                         FALSE, pool));
+  SVN_ERR (svn_opt_args_to_target_array2 (&targets, os, 
+                                          opt_state->targets, pool));
 
   if (opt_state->revprop)  /* operate on a revprop */
     {
       svn_revnum_t rev;
-      const char *URL, *target;
+      const char *URL;
       svn_string_t *propval;
-      const char *new_propval;
       const char *temp_dir;
 
-      /* All property commands insist on a specific revision when
-         operating on a revprop. */
-      if (opt_state->start_revision.kind == svn_opt_revision_unspecified)
-        return svn_cl__revprop_no_rev_error (pool);
-
-      /* Else some revision was specified, so proceed. */
-
       /* Implicit "." is okay for revision properties; it just helps
          us find the right repository. */
       svn_opt_push_implicit_dot_target (targets, pool);
 
-      /* Either we have a URL target, or an implicit wc-path ('.')
-         which needs to be converted to a URL. */
-      if (targets->nelts <= 0)
-        return svn_error_create(SVN_ERR_CL_INSUFFICIENT_ARGS, NULL,
-                                "No URL target available");
-      target = ((const char **) (targets->elts))[0];
-      SVN_ERR (svn_client_url_from_path (&URL, target, pool));  
-      if (URL == NULL)
-        return svn_error_create(SVN_ERR_UNVERSIONED_RESOURCE, NULL,
-                                "Either a URL or versioned item is required");
+      SVN_ERR (svn_cl__revprop_prepare (&opt_state->start_revision, targets,
+                                        &URL, pool));
 
       /* Fetch the current property. */
       SVN_ERR (svn_client_revprop_get (pname_utf8, &propval,
@@ -102,47 +84,45 @@ svn_cl__propedit (apr_getopt_t *os,
       /* Run the editor on a temporary file which contains the
          original property value... */
       SVN_ERR (svn_io_temp_dir (&temp_dir, pool));
-      SVN_ERR (svn_cl__edit_externally (&new_propval, NULL,
+      SVN_ERR (svn_cl__edit_externally (&propval, NULL,
                                         opt_state->editor_cmd, temp_dir,
-                                        propval->data, "svn-prop",
-                                        ctx->config, pool));
+                                        propval, "svn-prop",
+                                        ctx->config,
+                                        svn_prop_needs_translation (pname_utf8),
+                                        opt_state->encoding, pool));
       
       /* ...and re-set the property's value accordingly. */
-      if (new_propval)
+      if (propval)
         {
-          propval->data = new_propval;
-          propval->len = strlen (new_propval);
-
-          /* Possibly clean up the new propval before giving it to
-             svn_client_revprop_set. */
-          if (svn_prop_needs_translation (pname_utf8))
-            SVN_ERR (svn_subst_translate_string (&propval, propval,
-                                                 opt_state->encoding, pool));
-          else 
-            if (opt_state->encoding)
-              return svn_error_create 
-                (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
-                 "Bad encoding option: prop's value isn't stored as UTF8");
+          if (! svn_prop_needs_translation (pname_utf8)
+              && opt_state->encoding)
+            return svn_error_create 
+              (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
+               _("Bad encoding option: prop value not stored as UTF8"));
           
           SVN_ERR (svn_client_revprop_set (pname_utf8, propval,
                                            URL, &(opt_state->start_revision),
                                            &rev, opt_state->force, ctx, pool));
 
-          printf ("Set new value for property '%s' on revision %"
-                  SVN_REVNUM_T_FMT"\n", pname, rev);
+          SVN_ERR
+            (svn_cmdline_printf
+             (pool,
+              _("Set new value for property '%s' on revision %ld\n"),
+              pname_utf8, rev));
         }
       else
         {
-          printf ("No changes to property '%s' on revision %"
-                  SVN_REVNUM_T_FMT"\n", pname, rev);
+          SVN_ERR (svn_cmdline_printf
+                   (pool, _("No changes to property '%s' on revision %ld\n"),
+                    pname_utf8, rev));
         }
     }
   else if (opt_state->start_revision.kind != svn_opt_revision_unspecified)
     {
       return svn_error_createf
         (SVN_ERR_CL_ARG_PARSING_ERROR, NULL,
-         "Cannot specify revision for editing versioned property '%s'",
-         pname);
+         _("Cannot specify revision for editing versioned property '%s'"),
+         pname_utf8);
     }
   else  /* operate on a normal, versioned property (not a revprop) */
     {
@@ -167,7 +147,7 @@ svn_cl__propedit (apr_getopt_t *os,
         {
           return svn_error_create
             (SVN_ERR_CL_INSUFFICIENT_ARGS, NULL,
-             "Explicit target argument required");
+             _("Explicit target argument required"));
         }
 
       /* For each target, edit the property PNAME. */
@@ -176,31 +156,33 @@ svn_cl__propedit (apr_getopt_t *os,
           apr_hash_t *props;
           const char *target = ((const char **) (targets->elts))[i];
           svn_string_t *propval;
-          const char *new_propval;
           const char *base_dir = target;
-          const char *target_stdout;
+          const char *target_local;
           svn_wc_adm_access_t *adm_access;
           const svn_wc_entry_t *entry;
+          svn_opt_revision_t peg_revision;
           
           svn_pool_clear (subpool);
+          SVN_ERR (svn_cl__check_cancel (ctx->cancel_baton));
           if (svn_path_is_url (target))
             {
-              /* ### If/when svn_client_propset() supports setting
-                 properties remotely, this guard can go away.  Also,
-                 when we do that, we'll have to pass a real auth baton
-                 instead of NULL to svn_client_propget() below. */
+              /* ### If/when svn_client_propset2() supports setting
+                 properties remotely, this guard can go away. */
               return svn_error_createf
                 (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
-                 "Editing property on non-local target '%s' "
-                 "not yet supported", target);
+                 _("Editing property on non-local target '%s' "
+                 "not yet supported"), target);
             }
 
+          /* Propedits can only happen on HEAD or the working copy, so
+             the peg revision can be as unspecified. */
+          peg_revision.kind = svn_opt_revision_unspecified;
+          
           /* Fetch the current property. */
-          SVN_ERR (svn_client_propget (&props, pname_utf8, target,
-                                       &(opt_state->start_revision),
-                                       FALSE,
-                                       NULL,  /* ### pass ctx here */
-                                       subpool));
+          SVN_ERR (svn_client_propget2 (&props, pname_utf8, target,
+                                        &peg_revision,
+                                        &(opt_state->start_revision),
+                                        FALSE, ctx, subpool));
           
           /* Get the property value. */
           propval = apr_hash_get (props, target, APR_HASH_KEY_STRING);
@@ -208,58 +190,55 @@ svn_cl__propedit (apr_getopt_t *os,
             propval = svn_string_create ("", subpool);
           
           /* Split the path if it is a file path. */
-          SVN_ERR (svn_wc_adm_probe_open (&adm_access, NULL, target,
-                                          FALSE, FALSE, subpool));
+          SVN_ERR (svn_wc_adm_probe_open3 (&adm_access, NULL, target,
+                                           FALSE, 0, ctx->cancel_func,
+                                           ctx->cancel_baton, subpool));
           SVN_ERR (svn_wc_entry (&entry, target, adm_access, FALSE, subpool));
           if (! entry)
             return svn_error_createf
               (SVN_ERR_ENTRY_NOT_FOUND, NULL, 
-               "'%s' does not appear to be a working copy path", target);
+               _("'%s' does not appear to be a working copy path"), target);
           if (entry->kind == svn_node_file)
             svn_path_split (target, &base_dir, NULL, subpool);
           
           /* Run the editor on a temporary file which contains the
              original property value... */
-          SVN_ERR (svn_cl__edit_externally (&new_propval, NULL,
+          SVN_ERR (svn_cl__edit_externally (&propval, NULL,
                                             opt_state->editor_cmd,
                                             base_dir,
-                                            propval->data,
+                                            propval,
                                             "svn-prop",
                                             ctx->config,
+                                            svn_prop_needs_translation
+                                            (pname_utf8), opt_state->encoding,
                                             subpool));
           
-          SVN_ERR (svn_cmdline_path_local_style_from_utf8 (&target_stdout,
-                                                           target, subpool));
+          target_local = svn_path_local_style (target, subpool);
 
           /* ...and re-set the property's value accordingly. */
-          if (new_propval)
+          if (propval)
             {
-              propval->data = new_propval;
-              propval->len = strlen (new_propval);
-
-              /* Possibly clean up the new propval before giving it to
-                 svn_client_propset. */
-              if (svn_prop_needs_translation (pname_utf8))
-                SVN_ERR (svn_subst_translate_string (&propval, propval,
-                                                     opt_state->encoding,
-                                                     subpool));
-              else 
-                if (opt_state->encoding)
-                  return svn_error_create 
-                    (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
-                     "Bad encoding option: prop value not stored as UTF8");
+              if (! svn_prop_needs_translation (pname_utf8)
+                  && opt_state->encoding)
+                return svn_error_create 
+                  (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
+                   _("Bad encoding option: prop value not stored as UTF8"));
               
-              SVN_ERR (svn_client_propset (pname_utf8, propval, target, 
-                                           FALSE, subpool));
-              printf ("Set new value for property '%s' on '%s'\n",
-                      pname, target_stdout);
+              SVN_ERR (svn_client_propset2 (pname_utf8, propval, target, 
+                                            FALSE, opt_state->force,
+                                            ctx, subpool));
+              SVN_ERR
+                (svn_cmdline_printf
+                 (subpool, _("Set new value for property '%s' on '%s'\n"),
+                  pname_utf8, target_local));
             }
           else
             {
-              printf ("No changes to property '%s' on '%s'\n",
-                      pname, target_stdout);
+              SVN_ERR
+                (svn_cmdline_printf
+                 (subpool, _("No changes to property '%s' on '%s'\n"),
+                  pname_utf8, target_local));
             }
-          SVN_ERR (svn_cl__check_cancel (ctx->cancel_baton));
         }
       svn_pool_destroy (subpool);
     }
