@@ -21,15 +21,12 @@
 #include <apr_pools.h>
 #include <apr_hash.h>
 #include <apr_uuid.h>
-#include <apr_portable.h>
 
 #define APR_WANT_STDIO
 #define APR_WANT_STRFUNC
 #include <apr_want.h>
 
-#if APR_HAVE_STDLIB
 #include <stdlib.h>     /* for free() */
-#endif
 #include <assert.h>
 
 #include <ne_socket.h>
@@ -42,10 +39,11 @@
 #include "svn_delta.h"
 #include "svn_io.h"
 #include "svn_ra.h"
+#include "../libsvn_ra/ra_loader.h"
 #include "svn_path.h"
 #include "svn_xml.h"
 #include "svn_dav.h"
-#include "svn_base64.h"
+#include "svn_props.h"
 
 #include "svn_private_config.h"
 
@@ -79,9 +77,26 @@ typedef struct
 
 } resource_t;
 
+
+/* Context for parsing <D:error> bodies, used when we call ne_copy(). */
+struct copy_baton
+{
+  /* The method neon is about to execute. */
+  const char *method;
+  
+  /* A parser for handling <D:error> responses from mod_dav_svn. */
+  ne_xml_parser *error_parser;
+
+  /* If <D:error> is returned, here's where the parsed result goes. */
+  svn_error_t *err;
+
+  /* A place for allocating fields in this structure. */
+  apr_pool_t *pool;
+};
+
 typedef struct
 {
-  svn_ra_session_t *ras;
+  svn_ra_dav__session_t *ras;
   const char *activity_url;
 
   apr_hash_t *valid_targets;
@@ -101,6 +116,16 @@ typedef struct
   /* The commit callback and baton */
   svn_commit_callback_t callback;
   void *callback_baton;
+
+  /* The hash of lock-tokens owned by the working copy. */
+  apr_hash_t *tokens;
+
+  /* Whether or not to keep the locks after commit is done. */
+  svn_boolean_t keep_locks;
+
+  /* A context for neon COPY request callbacks. */
+  struct copy_baton *cb;
+
 } commit_ctx_t;
 
 typedef struct
@@ -119,6 +144,7 @@ typedef struct
   svn_boolean_t created; /* set if this is an add rather than an update */
   apr_pool_t *pool; /* the pool from open_foo() / add_foo() */
   put_baton_t *put_baton;  /* baton for this file's PUT request */
+  const char *token;       /* file's lock token, if available */
 } resource_baton_t;
 
 /* this property will be fetched from the server when we don't find it
@@ -148,7 +174,7 @@ static resource_t * dup_resource(resource_t *base, apr_pool_t *pool)
   return rsrc;
 }
 
-static svn_error_t * simple_request(svn_ra_session_t *ras, 
+static svn_error_t * simple_request(svn_ra_dav__session_t *ras, 
                                     const char *method,
                                     const char *url, 
                                     int *code,
@@ -183,11 +209,12 @@ static svn_error_t * simple_request(svn_ra_session_t *ras,
     }
 
   /* run the request and get the resulting status code (and svn_error_t) */
-  SVN_ERR( svn_ra_dav__request_dispatch(code, req, ras->sess,
-                                        method, url, okay_1, okay_2,
-                                        pool) );
-
-  return SVN_NO_ERROR;
+  return svn_ra_dav__request_dispatch(code, req, ras->sess,
+                                      method, url, okay_1, okay_2,
+#if SVN_NEON_0_25
+                                      NULL, NULL,
+#endif /* SVN_NEON_0_25 */
+                                      pool);
 }
 
 
@@ -262,9 +289,9 @@ static svn_error_t * get_version_url(commit_ctx_t *cc,
     {
       /* ### need a proper SVN_ERR here */
       return svn_error_create(APR_EGENERAL, NULL,
-                              "Could not fetch the Version Resource URL "
-                              "(needed during an import or when it is "
-                              "missing from the local, cached props)");
+                              _("Could not fetch the Version Resource URL "
+                                "(needed during an import or when it is "
+                                "missing from the local, cached props)"));
     }
 
   /* ensure we get the proper lifetime for this URL since it is going into
@@ -412,15 +439,38 @@ static svn_error_t * add_child(resource_t **child,
   return SVN_NO_ERROR;
 }
 
+#if SVN_NEON_0_25
+/* This implements the svn_ra_dav__request_interrogator() interface.
+   USERDATA is 'char **'. */
+static svn_error_t *interrogate_for_location(ne_request *request,
+                                             int dispatch_return_val,
+                                             void *userdata)
+{
+  char **location = userdata;
+
+  if (location)
+    {
+      const char *val = ne_get_response_header(request, "location");
+      if (val)
+        *location = strdup(val);
+    }
+
+  return SVN_NO_ERROR;
+}
+#endif /* SVN_NEON_0_25 */
+
+
 static svn_error_t * do_checkout(commit_ctx_t *cc,
                                  const char *vsn_url,
                                  svn_boolean_t allow_404,
+                                 const char *token,
                                  int *code,
                                  char **locn,
                                  apr_pool_t *pool)
 {
   ne_request *req;
   const char *body;
+  svn_error_t *err;
 
   /* assert: vsn_url != NULL */
 
@@ -432,7 +482,7 @@ static svn_error_t * do_checkout(commit_ctx_t *cc,
   if (req == NULL)
     {
       return svn_error_createf(SVN_ERR_RA_DAV_CREATING_REQUEST, NULL,
-                               "Could not create a CHECKOUT request (%s)",
+                               _("Could not create a CHECKOUT request (%s)"),
                                vsn_url);
     }
 
@@ -445,25 +495,40 @@ static svn_error_t * do_checkout(commit_ctx_t *cc,
                       "</D:activity-set></D:checkout>", cc->activity_url);
   ne_set_request_body_buffer(req, body, strlen(body));
 
+#ifndef SVN_NEON_0_25
   /* 
    * We have different const qualifiers here. locn is const char **,
    * but the prototype is void * (as opposed to const void *).
    */
   ne_add_response_header_handler(req, "location",
                                  ne_duplicate_header, (void *)locn);
+#endif /* ! SVN_NEON_0_25 */
+
+  if (token)
+    {
+      const char *token_header_val;
+      token_header_val = apr_psprintf(pool, "(<%s>)", token);
+      ne_add_request_header(req, "If", token_header_val);
+    }
 
   /* run the request and get the resulting status code (and svn_error_t) */
-  return svn_ra_dav__request_dispatch(code, req, cc->ras->sess,
-                                      "CHECKOUT", vsn_url,
-                                      201 /* Created */,
-                                      allow_404 ? 404 /* Not Found */ : 0,
-                                      pool);
+  err = svn_ra_dav__request_dispatch(code, req, cc->ras->sess,
+                                     "CHECKOUT", vsn_url,
+                                     201 /* Created */,
+                                     allow_404 ? 404 /* Not Found */ : 0,
+#if SVN_NEON_0_25
+                                     interrogate_for_location, locn,
+#endif /* SVN_NEON_0_25 */
+                                     pool);
+
+  return err;
 }
 
 
 static svn_error_t * checkout_resource(commit_ctx_t *cc,
                                        resource_t *rsrc,
-                                       svn_boolean_t allow_404, 
+                                       svn_boolean_t allow_404,
+                                       const char *token,
                                        apr_pool_t *pool)
 {
   int code;
@@ -478,7 +543,7 @@ static svn_error_t * checkout_resource(commit_ctx_t *cc,
     }
 
   /* check out the Version Resource */
-  err = do_checkout(cc, rsrc->vsn_url, allow_404, &code, &locn, pool);
+  err = do_checkout(cc, rsrc->vsn_url, allow_404, token, &code, &locn, pool);
 
   /* possibly run the request again, with a re-fetched Version Resource */
   if (err == NULL && allow_404 && code == 404)
@@ -491,7 +556,7 @@ static svn_error_t * checkout_resource(commit_ctx_t *cc,
       SVN_ERR( get_version_url(cc, rsrc, TRUE, pool) );
 
       /* do it again, but don't allow a 404 this time */
-      err = do_checkout(cc, rsrc->vsn_url, FALSE, &code, &locn, pool);
+      err = do_checkout(cc, rsrc->vsn_url, FALSE, token, &code, &locn, pool);
     }
 
   /* special-case when conflicts occur */
@@ -504,16 +569,16 @@ static svn_error_t * checkout_resource(commit_ctx_t *cc,
       if (err->apr_err == SVN_ERR_FS_CONFLICT)
         return svn_error_createf
           (err->apr_err, err,
-           "Your file or directory '%s' is probably out-of-date",
-           rsrc->local_path);
+           _("Your file or directory '%s' is probably out-of-date"),
+           svn_path_local_style(rsrc->local_path, pool));
       return err;
     }
 
   /* we got the header, right? */
   if (locn == NULL)
     return svn_error_create(SVN_ERR_RA_DAV_REQUEST_FAILED, NULL,
-                            "The CHECKOUT response did not contain a "
-                            "'Location:' header");
+                            _("The CHECKOUT response did not contain a "
+                              "'Location:' header"));
 
   /* The location is an absolute URI. We want just the path portion. */
   /* ### what to do with the rest? what if it points somewhere other
@@ -592,14 +657,26 @@ will see an error about 'cannot checkout out-of-date parent'.  Not
 really a big deal I guess.
 
 */
-static svn_error_t * do_proppatch(svn_ra_session_t *ras,
+static svn_error_t * do_proppatch(svn_ra_dav__session_t *ras,
                                   const resource_t *rsrc,
                                   resource_baton_t *rb,
                                   apr_pool_t *pool)
 {
   const char *url = rsrc->wr_url;
+  apr_hash_t *extra_headers = NULL;
+  
+  if (rb->token)
+    {
+      const char *token_header_val;
+      token_header_val = apr_psprintf(pool, "(<%s>)", rb->token);
+      
+      extra_headers = apr_hash_make(pool);
+      apr_hash_set(extra_headers, "If", APR_HASH_KEY_STRING,
+                   token_header_val);
+    }
+
   return svn_ra_dav__do_proppatch(ras, url, rb->prop_changes, 
-                                  rb->prop_deletes, pool);
+                                  rb->prop_deletes, extra_headers, pool);
 }
 
 
@@ -610,10 +687,11 @@ add_valid_target (commit_ctx_t *cc,
 {
   apr_hash_t *hash = cc->valid_targets;
   svn_string_t *path_str = svn_string_create(path, apr_hash_pool_get(hash));
-  apr_hash_set (hash, path_str->data, path_str->len, &kind);
+  apr_hash_set (hash, path_str->data, path_str->len, (void*)kind);
 }
 
 
+
 static svn_error_t * commit_open_root(void *edit_baton,
                                       svn_revnum_t base_revision,
                                       apr_pool_t *dir_pool,
@@ -647,6 +725,38 @@ static svn_error_t * commit_open_root(void *edit_baton,
   return SVN_NO_ERROR;
 }
 
+ 
+/* Helper func for commit_delete_entry.  Find all keys in LOCK_TOKENS
+   which are children of DIR.  Returns the keys (and their vals) in
+   CHILD_TOKENS.   No keys or values are reallocated or dup'd.  If no
+   keys are children, then return an empty hash.  Use POOL to allocate
+   new hash. */
+static apr_hash_t *get_child_tokens(apr_hash_t *lock_tokens,
+                                    const char *dir,
+                                    apr_pool_t *pool)
+{
+  apr_hash_index_t *hi;
+  apr_hash_t *tokens = apr_hash_make(pool);
+  apr_pool_t *subpool = svn_pool_create(pool);
+
+  for (hi = apr_hash_first(pool, lock_tokens); hi; hi = apr_hash_next(hi))
+    {
+      const void *key;
+      apr_ssize_t klen;
+      void *val;
+
+      svn_pool_clear(subpool);
+      apr_hash_this(hi, &key, &klen, &val);
+
+      if (svn_path_is_child(dir, key, subpool))
+        apr_hash_set(tokens, key, klen, val);
+    }
+
+  svn_pool_destroy(subpool);
+  return tokens;
+}
+
+
 static svn_error_t * commit_delete_entry(const char *path,
                                          svn_revnum_t revision,
                                          void *parent_baton,
@@ -657,28 +767,137 @@ static svn_error_t * commit_delete_entry(const char *path,
   apr_hash_t *extra_headers = NULL;
   const char *child;
   int code;
+  svn_error_t *serr;
 
   if (SVN_IS_VALID_REVNUM(revision))
     {
       const char *revstr = apr_psprintf(pool, "%ld", revision);
-      extra_headers = apr_hash_make(pool);
+
+      if (! extra_headers)
+        extra_headers = apr_hash_make(pool);
+
       apr_hash_set(extra_headers, SVN_DAV_VERSION_NAME_HEADER,
                    APR_HASH_KEY_STRING, revstr);
     }
 
   /* get the URL to the working collection */
-  SVN_ERR( checkout_resource(parent->cc, parent->rsrc, TRUE, pool) );
+  SVN_ERR( checkout_resource(parent->cc, parent->rsrc, TRUE, NULL, pool) );
 
   /* create the URL for the child resource */
   child = svn_path_url_add_component(parent->rsrc->wr_url, name, pool);
 
-  /* ### 404 is ignored, because mod_dav_svn is effectively merging
+  /* Start out assuming that we're deleting a file;  try to lookup the
+     path itself in the token-hash, and if found, attach it to the If:
+     header. */
+  if (parent->cc->tokens)
+    {
+      const char *token = 
+        apr_hash_get(parent->cc->tokens, path, APR_HASH_KEY_STRING);
+
+      if (token)
+        {
+          const char *token_header_val;
+          const char *token_uri;
+
+          token_uri = svn_path_url_add_component(parent->cc->ras->url,
+                                                 path, pool);
+          token_header_val = apr_psprintf(pool, "<%s> (<%s>)",
+                                          token_uri, token);
+          extra_headers = apr_hash_make(pool);
+          apr_hash_set(extra_headers, "If", APR_HASH_KEY_STRING,
+                       token_header_val);
+        }
+    }
+
+  /* dav_method_delete() always calls dav_unlock(), but if the svn
+     client passed --no-unlock to 'svn commit', then we need to send a
+     header which prevents mod_dav_svn from actually doing the unlock. */
+  if (parent->cc->keep_locks)
+    {
+      if (! extra_headers)
+        extra_headers = apr_hash_make(pool);
+
+      apr_hash_set(extra_headers, SVN_DAV_OPTIONS_HEADER,
+                   APR_HASH_KEY_STRING, SVN_DAV_OPTION_KEEP_LOCKS);
+    }
+
+  /* 404 is ignored, because mod_dav_svn is effectively merging
      against the HEAD revision on-the-fly.  In such a universe, a
      failed deletion (because it's already missing) is OK;  deletion
      is an idempotent merge operation. */
-  SVN_ERR( simple_request(parent->cc->ras, "DELETE", child, &code,
-                          extra_headers,
-                          204 /* Created */, 404 /* Not Found */, pool) );
+  serr =  simple_request(parent->cc->ras, "DELETE", child, &code,
+                         extra_headers,
+                         204 /* Created */, 404 /* Not Found */, pool);
+
+  /* A locking-related error most likely means we were deleting a
+     directory rather than a file, and didn't send all of the
+     necessary lock-tokens within the directory. */
+  if (serr && ((serr->apr_err == SVN_ERR_FS_BAD_LOCK_TOKEN)
+               || (serr->apr_err == SVN_ERR_FS_NO_LOCK_TOKEN)
+               || (serr->apr_err == SVN_ERR_FS_LOCK_OWNER_MISMATCH)
+               || (serr->apr_err == SVN_ERR_FS_PATH_ALREADY_LOCKED)))
+    {
+      /* Re-attempt the DELETE request as if the path were a directory.
+         Discover all lock-tokens within the directory, and send them in
+         the body of the request (which is normally empty).  Of course,
+         if we don't *find* any additional lock-tokens, don't bother to
+         retry (it ain't gonna do any good).
+         
+         Note that we're not sending the locks in the If: header, for
+         the same reason we're not sending in MERGE's headers: httpd has
+       limits on the amount of data it's willing to receive in headers. */
+      
+      apr_hash_t *child_tokens = NULL;
+      ne_request *req;
+      const char *body;
+      const char *token;
+      svn_stringbuf_t *locks_list;
+      
+      if (parent->cc->tokens)
+        child_tokens = get_child_tokens(parent->cc->tokens, path, pool);
+      
+      /* No kiddos?  Return the original error.  Else, clear it so it
+         doesn't get leaked.  */
+      if ((! child_tokens) 
+          || (apr_hash_count(child_tokens) == 0))
+        return serr;
+      else
+        svn_error_clear(serr);
+      
+      /* In preparation of directory locks, go ahead and add the actual
+         target's lock token to those of its children. */
+      if ((token = apr_hash_get(parent->cc->tokens, path, 
+                                APR_HASH_KEY_STRING)))
+        apr_hash_set(child_tokens, path, APR_HASH_KEY_STRING, token);
+      
+      SVN_ERR (svn_ra_dav__assemble_locktoken_body(&locks_list,
+                                                   child_tokens, pool));
+      
+      req = ne_request_create(parent->cc->ras->sess, "DELETE", child);
+      if (req == NULL)
+        return svn_error_createf(SVN_ERR_RA_DAV_CREATING_REQUEST, NULL,
+                                 _("Could not create a DELETE request (%s)"),
+                                 child);
+      
+      body = apr_psprintf(pool, 
+                          "<?xml version=\"1.0\" encoding=\"utf-8\"?> %s",
+                          locks_list->data);
+      ne_set_request_body_buffer(req, body, strlen(body));
+      
+      /* Don't use SVN_ERR() here because some preprocessors can't
+         handle a compile-time conditional inside a macro call. */
+      serr = svn_ra_dav__request_dispatch(&code, req, parent->cc->ras->sess,
+                                          "DELETE", child,
+                                          204 /* Created */,
+                                          404 /* Not Found */,
+#if SVN_NEON_0_25
+                                          NULL, NULL,
+#endif /* SVN_NEON_0_25 */
+                                          pool);
+      SVN_ERR(serr);
+    }
+  else if (serr)
+    return serr;
 
   /* Add this path to the valid targets hash. */
   add_valid_target (parent->cc, path, svn_nonrecursive);
@@ -686,6 +905,46 @@ static svn_error_t * commit_delete_entry(const char *path,
   return SVN_NO_ERROR;
 }
 
+
+
+/* A callback of type ne_create_request_fn;  called whenever neon
+   creates a request. */
+static void 
+create_request_hook(ne_request *req,
+                    void *userdata,
+                    const char *method,
+                    const char *requri)
+{
+  struct copy_baton *cb = userdata;
+
+  if (strcmp(method, "COPY") == 0)
+    cb->method = apr_pstrdup(cb->pool, method);
+  else
+    cb->method = NULL;
+}
+
+
+/* A callback of type ne_pre_send_fn;  called whenever neon is just
+   about to send a COPY request. */
+static void
+pre_send_hook(ne_request *req,
+              void *userdata,
+              ne_buffer *header)
+{
+  struct copy_baton *cb = userdata;
+
+  if (! cb->method)
+    return;
+
+  if (strcmp(cb->method, "COPY") == 0)
+    {
+      cb->error_parser = ne_xml_create();
+      svn_ra_dav__add_error_handler(req, cb->error_parser,
+                                    &(cb->err), cb->pool);
+    }
+}
+
+
 static svn_error_t * commit_add_dir(const char *path,
                                     void *parent_baton,
                                     const char *copyfrom_path,
@@ -702,7 +961,7 @@ static svn_error_t * commit_add_dir(const char *path,
 
   /* check out the parent resource so that we can create the new collection
      as one of its children. */
-  SVN_ERR( checkout_resource(parent->cc, parent->rsrc, TRUE, dir_pool) );
+  SVN_ERR( checkout_resource(parent->cc, parent->rsrc, TRUE, NULL, dir_pool) );
 
   /* create a child object that contains all the resource urls */
   child = apr_pcalloc(dir_pool, sizeof(*child));
@@ -754,12 +1013,26 @@ static svn_error_t * commit_add_dir(const char *path,
                        copy_src,            /* source URI */
                        child->rsrc->wr_url); /* dest URI */
 
+      /* Did we get a <D:error> response? */
+      if (parent->cc->cb->err)
+        {
+          if (parent->cc->cb->error_parser)
+            ne_xml_destroy(parent->cc->cb->error_parser);
+          return parent->cc->cb->err;
+        }
+
+      /* Did we get some error from neon? */
       if (status != NE_OK)
         {
           const char *msg = apr_psprintf(dir_pool, "COPY of %s", path);
+          if (parent->cc->cb->error_parser)
+            ne_xml_destroy(parent->cc->cb->error_parser);
           return svn_ra_dav__convert_error(parent->cc->ras->sess,
                                            msg, status, workpool);
         }
+
+      if (parent->cc->cb->error_parser)
+        ne_xml_destroy(parent->cc->cb->error_parser);
     }
 
   /* Add this path to the valid targets hash. */
@@ -817,7 +1090,7 @@ static svn_error_t * commit_change_dir_prop(void *dir_baton,
   record_prop_change(dir->pool, dir, name, value);
 
   /* do the CHECKOUT sooner rather than later */
-  SVN_ERR( checkout_resource(dir->cc, dir->rsrc, TRUE, pool) );
+  SVN_ERR( checkout_resource(dir->cc, dir->rsrc, TRUE, NULL, pool) );
 
   /* Add this path to the valid targets hash. */
   add_valid_target (dir->cc, dir->rsrc->local_path, svn_nonrecursive);
@@ -860,7 +1133,7 @@ static svn_error_t * commit_add_file(const char *path,
   */
 
   /* Do the parent CHECKOUT first */
-  SVN_ERR( checkout_resource(parent->cc, parent->rsrc, TRUE, workpool) );
+  SVN_ERR( checkout_resource(parent->cc, parent->rsrc, TRUE, NULL, workpool) );
 
   /* Construct a file_baton that contains all the resource urls. */
   file = apr_pcalloc(file_pool, sizeof(*file));
@@ -870,7 +1143,9 @@ static svn_error_t * commit_add_file(const char *path,
   SVN_ERR( add_child(&rsrc, parent->cc, parent->rsrc,
                      name, 1, SVN_INVALID_REVNUM, workpool) );
   file->rsrc = dup_resource(rsrc, file_pool);
-
+  if (parent->cc->tokens)
+    file->token = apr_hash_get(parent->cc->tokens, path, APR_HASH_KEY_STRING);
+  
   /* If the parent directory existed before this commit then there may be a
      file with this URL already. We need to ensure such a file does not
      exist, which we do by attempting a PROPFIND.  Of course, a
@@ -952,12 +1227,26 @@ static svn_error_t * commit_add_file(const char *path,
                        copy_src,            /* source URI */
                        file->rsrc->wr_url); /* dest URI */
 
+      /* Did we get a <D:error> response? */
+      if (parent->cc->cb->err)
+        {
+          if (parent->cc->cb->error_parser)
+            ne_xml_destroy(parent->cc->cb->error_parser);
+          return parent->cc->cb->err;
+        }
+
+      /* Did we get some error from neon? */
       if (status != NE_OK)
         {
           const char *msg = apr_psprintf(file_pool, "COPY of %s", path);
+          if (parent->cc->cb->error_parser)
+            ne_xml_destroy(parent->cc->cb->error_parser);
           return svn_ra_dav__convert_error(parent->cc->ras->sess,
                                            msg, status, workpool);
         }
+      
+      if (parent->cc->cb->error_parser)
+        ne_xml_destroy(parent->cc->cb->error_parser);
     }
 
   /* Add this path to the valid targets hash. */
@@ -989,9 +1278,12 @@ static svn_error_t * commit_open_file(const char *path,
   SVN_ERR( add_child(&rsrc, parent->cc, parent->rsrc,
                      name, 0, base_revision, workpool) );
   file->rsrc = dup_resource(rsrc, file_pool);
+  if (parent->cc->tokens)
+    file->token = apr_hash_get(parent->cc->tokens, path, APR_HASH_KEY_STRING);
 
   /* do the CHECKOUT now. we'll PUT the new file contents later on. */
-  SVN_ERR( checkout_resource(parent->cc, file->rsrc, TRUE, workpool) );
+  SVN_ERR( checkout_resource(parent->cc, file->rsrc, TRUE,
+                             file->token, workpool) );
 
   /* ### wait for apply_txdelta before doing a PUT. it might arrive a
      ### "long time" from now. certainly after many other operations, so
@@ -1076,7 +1368,7 @@ static svn_error_t * commit_change_file_prop(void *file_baton,
   record_prop_change(file->pool, file, name, value);
 
   /* do the CHECKOUT sooner rather than later */
-  SVN_ERR( checkout_resource(file->cc, file->rsrc, TRUE, pool) );
+  SVN_ERR( checkout_resource(file->cc, file->rsrc, TRUE, file->token, pool) );
 
   /* Add this path to the valid targets hash. */
   add_valid_target (file->cc, file->rsrc->local_path, svn_nonrecursive);
@@ -1111,6 +1403,18 @@ static svn_error_t * commit_close_file(void *file_baton,
       
       ne_add_request_header(req, "Content-Type", SVN_SVNDIFF_MIME_TYPE);
       
+      if (file->token)
+        {
+          const char *token_header_val;
+          const char *token_uri;
+
+          token_uri = svn_path_url_add_component(cc->ras->url,
+                                                 file->rsrc->url, pool);
+          token_header_val = apr_psprintf(pool, "<%s> (<%s>)",
+                                          token_uri, file->token);
+          ne_add_request_header(req, "If", token_header_val);
+        }
+
       if (pb->base_checksum)
         ne_add_request_header
           (req, SVN_DAV_BASE_FULLTEXT_MD5_HEADER, pb->base_checksum);
@@ -1132,6 +1436,9 @@ static svn_error_t * commit_close_file(void *file_baton,
       err = svn_ra_dav__request_dispatch(&code, req, sess, "PUT", url,
                                          201 /* Created */,
                                          204 /* No Content */,
+#if SVN_NEON_0_25
+                                         NULL, NULL,
+#endif /* SVN_NEON_0_25 */
                                          pool);
       
       /* we're done with the file.  this should delete it. */
@@ -1164,6 +1471,8 @@ static svn_error_t * commit_close_edit(void *edit_baton,
                                       cc->ras->root.path,
                                       cc->activity_url,
                                       cc->valid_targets,
+                                      cc->tokens,
+                                      cc->keep_locks,
                                       cc->disable_merge_response,
                                       pool) );
   SVN_ERR( delete_activity(edit_baton, pool) );
@@ -1221,7 +1530,7 @@ static svn_error_t * apply_log_message(commit_ctx_t *cc,
     
     /* To set the log message, we must checkout the latest baseline
        and get back a mutable "working" baseline.  */
-    err = checkout_resource(cc, &baseline_rsrc, FALSE, pool);
+    err = checkout_resource(cc, &baseline_rsrc, FALSE, NULL, pool);
 
     /* There's a small chance of a race condition here, if apache is
        experiencing heavy commit concurrency or if the network has
@@ -1260,17 +1569,24 @@ static svn_error_t * apply_log_message(commit_ctx_t *cc,
   return SVN_NO_ERROR;
 }
 
-svn_error_t * svn_ra_dav__get_commit_editor(void *session_baton,
+svn_error_t * svn_ra_dav__get_commit_editor(svn_ra_session_t *session,
                                             const svn_delta_editor_t **editor,
                                             void **edit_baton,
                                             const char *log_msg,
                                             svn_commit_callback_t callback,
                                             void *callback_baton,
+                                            apr_hash_t *lock_tokens,
+                                            svn_boolean_t keep_locks,
                                             apr_pool_t *pool)
 {
-  svn_ra_session_t *ras = session_baton;
+  svn_ra_dav__session_t *ras = session->priv;
   svn_delta_editor_t *commit_editor;
   commit_ctx_t *cc;
+  struct copy_baton *cb;
+
+  /* Build a copy_baton for COPY requests. */
+  cb = apr_pcalloc(pool, sizeof(*cb));
+  cb->pool = pool;
 
   /* Build the main commit editor's baton. */
   cc = apr_pcalloc(pool, sizeof(*cc));
@@ -1282,6 +1598,9 @@ svn_error_t * svn_ra_dav__get_commit_editor(void *session_baton,
   cc->log_msg = log_msg;
   cc->callback = callback;
   cc->callback_baton = callback_baton;
+  cc->tokens = lock_tokens;
+  cc->keep_locks = keep_locks;
+  cc->cb = cb;
 
   /* If the caller didn't give us any way of storing wcprops, then
      there's no point in getting back a MERGE response full of VR's. */
@@ -1304,6 +1623,12 @@ svn_error_t * svn_ra_dav__get_commit_editor(void *session_baton,
   */
   SVN_ERR( apply_log_message(cc, log_msg, pool) );
 
+  /* Register request hooks in the neon session.  They specifically
+     allow any COPY requests (ne_copy()) to parse <D:error>
+     responses.  They're no-ops for other requests. */
+  ne_hook_create_request(ras->sess, create_request_hook, cb);
+  ne_hook_pre_send(ras->sess, pre_send_hook, cb);
+
   /*
   ** Set up the editor.
   **
