@@ -24,6 +24,7 @@
 #include <ne_socket.h>
 #include <ne_uri.h>
 #include <ne_compress.h>
+#include <ne_basic.h>
 
 #include "svn_pools.h"
 #include "svn_path.h"
@@ -306,8 +307,28 @@ static int ra_dav_error_accepter(void *userdata,
                                  ne_request *req,
                                  const ne_status *st)
 {
+  /* Before, this function was being run for *all* responses including
+     the 401 auth challenge.  In neon 0.24.x that was harmless.  But
+     in neon 0.25.0, trying to parse a 401 response as XML using
+     ne_xml_parse_v aborts the response; so the auth hooks never got a
+     chance. */
+#if SVN_NEON_0_25
+  ne_content_type ctype;
+
+  /* Only accept non-2xx responses with text/xml content-type */
+  if (st->klass != 2 && ne_get_content_type(req, &ctype) == 0)
+    {
+      int is_xml = 
+        (strcmp(ctype.type, "text") == 0 && strcmp(ctype.subtype, "xml") == 0);
+      ne_free(ctype.value);        
+      return is_xml;
+    }
+  else 
+    return 0;
+#else /* ! SVN_NEON_0_25 */
   /* Only accept the body-response if the HTTP status code is *not* 2XX. */
   return (st->klass != 2);
+#endif /* if/else SVN_NEON_0_25 */
 }
 
 
@@ -483,7 +504,12 @@ typedef struct spool_reader_baton_t
 } spool_reader_baton_t;
 
 
-static void 
+/* This implements the ne_block_reader() callback interface. */
+#if SVN_NEON_0_25
+static int
+#else /* ! SVN_NEON_0_25 */
+static void
+#endif /* if/else SVN_NEON_0_25 */
 spool_reader(void *userdata, 
              const char *buf, 
              size_t len)
@@ -492,6 +518,14 @@ spool_reader(void *userdata,
   if (! baton->error)
     baton->error = svn_io_file_write_full(baton->spool_file, buf, 
                                           len, NULL, baton->pool);
+
+#if SVN_NEON_0_25
+  if (baton->error)
+    /* ### Call ne_set_error(), as ne_block_reader doc implies? */
+    return 1;
+  else
+    return 0;
+#endif /* SVN_NEON_0_25 */
 }
 
 
@@ -556,7 +590,9 @@ parsed_request(ne_session *sess,
   ne_xml_parser *success_parser = NULL;
   ne_xml_parser *error_parser = NULL;
   int rv;
+#ifndef SVN_NEON_0_25
   int decompress_rv;
+#endif /* ! SVN_NEON_0_25 */
   int code;
   int expected_code;
   const char *msg;
@@ -639,9 +675,11 @@ parsed_request(ne_session *sess,
         goto cleanup;
 
       tmpfile_path = svn_path_join(tmpfile_path, "dav-spool", pool);
-      SVN_ERR (svn_io_open_unique_file (&spool_reader_baton.spool_file, 
-                                        &spool_reader_baton.spool_file_name,
-                                        tmpfile_path, "", FALSE, pool));
+      err = svn_io_open_unique_file (&spool_reader_baton.spool_file, 
+                                     &spool_reader_baton.spool_file_name,
+                                     tmpfile_path, "", FALSE, pool);
+      if (err)
+        goto cleanup;
       spool_reader_baton.pool = pool;
       spool_reader_baton.error = SVN_NO_ERROR;
       if (ras->compression)
@@ -689,6 +727,13 @@ parsed_request(ne_session *sess,
         }
     }
 
+#if SVN_NEON_0_25
+  if (decompress_main)
+    ne_decompress_destroy(decompress_main);
+
+  if (decompress_err)
+    ne_decompress_destroy(decompress_err);
+#else  /* ! SVN_NEON_0_25 */
   if (decompress_main)
     {
       decompress_rv = ne_decompress_destroy(decompress_main);
@@ -706,6 +751,7 @@ parsed_request(ne_session *sess,
           rv = decompress_rv;
         }
     }
+#endif /* if/else SVN_NEON_0_25 */
   
   code = ne_get_status(req)->code;
   if (status_code)
@@ -850,6 +896,25 @@ svn_ra_dav__maybe_store_auth_info(svn_ra_dav__session_t *ras)
 }
 
 
+svn_error_t *
+svn_ra_dav__maybe_store_auth_info_after_result(svn_error_t *err,
+                                               svn_ra_dav__session_t *ras)
+{
+  if (! err || (err->apr_err != SVN_ERR_RA_NOT_AUTHORIZED))
+    {
+      svn_error_t *err2 = svn_ra_dav__maybe_store_auth_info(ras);
+      if (err2 && ! err)
+        return err2;
+      else if (err)
+        {
+          svn_error_clear(err2);
+          return err;          
+        }
+    }
+
+  return err;
+}
+
 
 void
 svn_ra_dav__add_error_handler(ne_request *request,
@@ -881,6 +946,10 @@ svn_ra_dav__request_dispatch(int *code_p,
                              const char *url,
                              int okay_1,
                              int okay_2,
+#if SVN_NEON_0_25
+                             svn_ra_dav__request_interrogator interrogator,
+                             void *interrogator_baton,
+#endif /* SVN_NEON_0_25 */
                              apr_pool_t *pool)
 {
   ne_xml_parser *error_parser;
@@ -890,6 +959,9 @@ svn_ra_dav__request_dispatch(int *code_p,
   int code;
   const char *msg;
   svn_error_t *err = SVN_NO_ERROR;
+#if SVN_NEON_0_25
+  svn_error_t *err2 = SVN_NO_ERROR;
+#endif /* SVN_NEON_0_25 */
 
   /* attach a standard <D:error> body parser to the request */
   error_parser = ne_xml_create();
@@ -908,9 +980,23 @@ svn_ra_dav__request_dispatch(int *code_p,
   if (code_p)
      *code_p = code;
 
+#if SVN_NEON_0_25
+  if (interrogator)
+    err2 = (*interrogator)(request, rv, interrogator_baton);
+#endif /* SVN_NEON_0_25 */
+
   ne_request_destroy(request);
   ne_xml_destroy(error_parser);
 
+#if SVN_NEON_0_25
+  /* If the request interrogator returned error, pass that along now. */
+  if (err2)
+    {
+      svn_error_clear(err);
+      return err2;
+    }
+#endif /* SVN_NEON_0_25 */
+
   /* If the status code was one of the two that we expected, then go
      ahead and return now. IGNORE any marshalled error. */
   if (rv == NE_OK && (code == okay_1 || code == okay_2))
