@@ -1,7 +1,7 @@
 /* commit.c --- editor for committing changes to a filesystem.
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2006 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -145,8 +145,8 @@ check_authz (struct edit_baton *editor_baton, const char *path,
     {
       svn_boolean_t allowed;
 
-      editor_baton->authz_callback (required, &allowed, root, path,
-                                    editor_baton->authz_baton, pool);
+      SVN_ERR (editor_baton->authz_callback (required, &allowed, root, path,
+                                             editor_baton->authz_baton, pool));
       if (!allowed)
         return svn_error_create(required & svn_authz_write ?
                                 SVN_ERR_AUTHZ_UNWRITABLE :
@@ -208,9 +208,6 @@ open_root (void *edit_baton,
   SVN_ERR (svn_fs_txn_name (&(eb->txn_name), eb->txn, eb->pool));
   SVN_ERR (svn_fs_txn_root (&(eb->txn_root), eb->txn, eb->pool));
 
-  /* Check read access to root */
-  SVN_ERR(check_authz (eb, "/", eb->txn_root, svn_authz_read, pool));
-
   /* Create a root dir baton.  The `base_path' field is an -absolute-
      path in the filesystem, upon which all further editor paths are
      based. */
@@ -388,10 +385,6 @@ open_directory (const char *path,
   svn_node_kind_t kind;
   const char *full_path = svn_path_join (eb->base_path, path, pool);
 
-  /* Check for read authorization. */
-  SVN_ERR (check_authz (eb, full_path, eb->txn_root,
-                        svn_authz_read, pool));
-
   /* Check PATH in our transaction.  If it does not exist,
      return a 'Path not present' error. */
   SVN_ERR (svn_fs_check_path (&kind, eb->txn_root, full_path, pool));
@@ -786,8 +779,8 @@ svn_repos_get_commit_editor3 (const svn_delta_editor_t **editor,
     {
       svn_boolean_t allowed;
 
-      authz_callback (svn_authz_write, &allowed, NULL, NULL,
-                      authz_baton, pool);
+      SVN_ERR (authz_callback (svn_authz_write, &allowed, NULL, NULL,
+                               authz_baton, pool));
       if (!allowed)
         return svn_error_create(SVN_ERR_AUTHZ_UNWRITABLE, NULL,
                                 "Not authorized to open a commit editor.");
