@@ -2,7 +2,7 @@
  * config.c :  reading configuration information
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -22,11 +22,14 @@
 #define APR_WANT_MEMFUNC
 #include <apr_want.h>
 
+#include <apr_general.h>
 #include <apr_lib.h>
-#include <apr_user.h>
 #include "svn_error.h"
+#include "svn_pools.h"
 #include "config_impl.h"
 
+#include "svn_private_config.h"
+
 
 
 
@@ -88,12 +91,12 @@ svn_config_read (svn_config_t **cfgp, const char *file,
      Subversion-specific and Windows-specific.  Even if we eventually
      want to have APR offer a generic config-reading interface, it
      makes sense to test it here first and migrate it later. */
-#ifdef SVN_WIN32
+#ifdef WIN32
   if (0 == strncmp (file, SVN_REGISTRY_PREFIX, SVN_REGISTRY_PREFIX_LEN))
     err = svn_config__parse_registry (cfg, file + SVN_REGISTRY_PREFIX_LEN,
                                       must_exist);
   else
-#endif /* SVN_WIN32 */
+#endif /* WIN32 */
     err = svn_config__parse_file (cfg, file, must_exist);
 
   if (err != SVN_NO_ERROR)
@@ -109,11 +112,11 @@ svn_config_read (svn_config_t **cfgp, const char *file,
 /* Read various configuration sources into *CFGP, in this order, with
  * later reads overriding the results of earlier ones:
  *
- *    1. SYS_REGISTRY_PATH   (only on SVN_WIN32, but ignored if NULL)
+ *    1. SYS_REGISTRY_PATH   (only on Win32, but ignored if NULL)
  *
  *    2. SYS_FILE_PATH       (everywhere, but ignored if NULL)
  *
- *    3. USR_REGISTRY_PATH   (only on SVN_WIN32, but ignored if NULL)
+ *    3. USR_REGISTRY_PATH   (only on Win32, but ignored if NULL)
  *
  *    4. USR_FILE_PATH       (everywhere, but ignored if NULL)
  *
@@ -132,13 +135,13 @@ read_all (svn_config_t **cfgp,
 
   /*** Read system-wide configurations first... ***/
 
-#ifdef SVN_WIN32
+#ifdef WIN32
   if (sys_registry_path)
     {
       SVN_ERR (svn_config_read (cfgp, sys_registry_path, FALSE, pool));
       red_config = TRUE;
     }
-#endif /* SVN_WIN32 */
+#endif /* WIN32 */
 
   if (sys_file_path)
     {
@@ -153,7 +156,7 @@ read_all (svn_config_t **cfgp,
 
   /*** ...followed by per-user configurations. ***/
 
-#ifdef SVN_WIN32
+#ifdef WIN32
   if (usr_registry_path)
     {
       if (red_config)
@@ -164,7 +167,7 @@ read_all (svn_config_t **cfgp,
           red_config = TRUE;
         }
     }
-#endif /* SVN_WIN32 */
+#endif /* WIN32 */
 
   if (usr_file_path)
     {
@@ -186,6 +189,7 @@ read_all (svn_config_t **cfgp,
 
 static svn_error_t *
 get_category_config (svn_config_t **cfg,
+                     const char *config_dir,
                      const char *category,
                      apr_pool_t *pool)
 {
@@ -194,15 +198,22 @@ get_category_config (svn_config_t **cfg,
   
   *cfg = NULL;
 
-#ifdef SVN_WIN32
-  sys_reg_path = apr_pstrcat (pool, SVN_REGISTRY_SYS_CONFIG_PATH,
-                              category, NULL);
-  usr_reg_path = apr_pstrcat (pool, SVN_REGISTRY_USR_CONFIG_PATH,
-                              category, NULL);
-#endif /* SVN_WIN32 */
+  if (! config_dir)
+    {
+#ifdef WIN32
+      sys_reg_path = apr_pstrcat (pool, SVN_REGISTRY_SYS_CONFIG_PATH,
+                                  category, NULL);
+      usr_reg_path = apr_pstrcat (pool, SVN_REGISTRY_USR_CONFIG_PATH,
+                                  category, NULL);
+#endif /* WIN32 */
 
-  SVN_ERR (svn_config__sys_config_path (&sys_cfg_path, category, pool));
-  SVN_ERR (svn_config__user_config_path (&usr_cfg_path, category, pool));
+      SVN_ERR (svn_config__sys_config_path (&sys_cfg_path, category, pool));
+    }
+  else
+    sys_cfg_path = NULL;
+
+  SVN_ERR (svn_config__user_config_path (config_dir, &usr_cfg_path, category,
+                                         pool));
   SVN_ERR (read_all (cfg,
                      sys_reg_path, usr_reg_path,
                      sys_cfg_path, usr_cfg_path,
@@ -214,19 +225,22 @@ get_category_config (svn_config_t **cfg,
 
 svn_error_t *
 svn_config_get_config (apr_hash_t **cfg_hash,
+                       const char *config_dir,
                        apr_pool_t *pool)
 {
   svn_config_t *cfg;
   *cfg_hash = apr_hash_make (pool);
   
 #define CATLEN (sizeof (SVN_CONFIG_CATEGORY_SERVERS) - 1)
-  SVN_ERR (get_category_config (&cfg, SVN_CONFIG_CATEGORY_SERVERS, pool));
+  SVN_ERR (get_category_config (&cfg, config_dir, SVN_CONFIG_CATEGORY_SERVERS,
+                                pool));
   if (cfg)
     apr_hash_set (*cfg_hash, SVN_CONFIG_CATEGORY_SERVERS, CATLEN, cfg);
 #undef CATLEN
 
 #define CATLEN (sizeof (SVN_CONFIG_CATEGORY_CONFIG) - 1)
-  SVN_ERR (get_category_config (&cfg, SVN_CONFIG_CATEGORY_CONFIG, pool));
+  SVN_ERR (get_category_config (&cfg, config_dir, SVN_CONFIG_CATEGORY_CONFIG,
+                                pool));
   if (cfg)
     apr_hash_set (*cfg_hash, SVN_CONFIG_CATEGORY_CONFIG, CATLEN, cfg);
 #undef CATLEN
@@ -432,9 +446,9 @@ make_string_from_option (const char **valuep, svn_config_t *cfg,
 #define FMT_END_LEN   (sizeof (FMT_END) - 1)
 
 
-/* Expand OPT_VALUE in SECTION to *OPT_X_VALUE. If no variable
-   replacements are done, set OPT_X_VALUE to NULL. Allocate from
-   X_POOL */
+/* Expand OPT_VALUE (which may be NULL) in SECTION into *OPT_X_VALUEP.
+   If no variable replacements are done, set *OPT_X_VALUEP to
+   NULL. Allocate from X_POOL. */
 static void
 expand_option_value (svn_config_t *cfg, cfg_section_t *section,
                      const char *opt_value, const char **opt_x_valuep,
@@ -592,6 +606,115 @@ svn_config_set (svn_config_t *cfg,
 }
 
 
+
+svn_error_t *
+svn_config_get_bool (svn_config_t *cfg, svn_boolean_t *valuep,
+                     const char *section, const char *option,
+                     svn_boolean_t default_value)
+{
+  const char *tmp_value;
+
+  svn_config_get (cfg, &tmp_value, section, option, NULL);
+  if (tmp_value == NULL)
+    *valuep = default_value;
+  else if (0 == strcasecmp (tmp_value, SVN_CONFIG_TRUE)
+           || 0 == strcasecmp (tmp_value, "yes")
+           || 0 == strcasecmp (tmp_value, "on")
+           || 0 == strcmp (tmp_value, "1"))
+    *valuep = TRUE;
+  else if (0 == strcasecmp (tmp_value, SVN_CONFIG_FALSE)
+           || 0 == strcasecmp (tmp_value, "no")
+           || 0 == strcasecmp (tmp_value, "off")
+           || 0 == strcmp (tmp_value, "0"))
+    *valuep = FALSE;
+  else
+    return svn_error_createf (SVN_ERR_RA_DAV_INVALID_CONFIG_VALUE, NULL,
+                              _("Config error: invalid boolean value '%s'"),
+                              tmp_value);
+
+  return SVN_NO_ERROR;
+}
+
+
+
+void
+svn_config_set_bool (svn_config_t *cfg,
+                     const char *section, const char *option,
+                     svn_boolean_t value)
+{
+  svn_config_set (cfg, section, option,
+                  (value ? SVN_CONFIG_TRUE : SVN_CONFIG_FALSE));
+}
+
+
+
+int
+svn_config__enumerate_sections (svn_config_t *cfg,
+                               svn_config__section_enumerator_t callback,
+                                void *baton)
+{
+  return svn_config_enumerate_sections (cfg,
+           (svn_config_section_enumerator_t) callback, baton);
+}
+
+int
+svn_config_enumerate_sections (svn_config_t *cfg,
+                               svn_config_section_enumerator_t callback,
+                               void *baton)
+{
+  apr_hash_index_t *sec_ndx;
+  int count = 0;
+  apr_pool_t *subpool = svn_pool_create (cfg->x_pool);
+
+  for (sec_ndx = apr_hash_first (subpool, cfg->sections);
+       sec_ndx != NULL;
+       sec_ndx = apr_hash_next (sec_ndx))
+    {
+      void *sec_ptr;
+      cfg_section_t *sec;
+
+      apr_hash_this (sec_ndx, NULL, NULL, &sec_ptr);
+      sec = sec_ptr;
+      ++count;
+      if (!callback (sec->name, baton))
+        break;
+    }
+
+  svn_pool_destroy (subpool);
+  return count;
+}
+
+
+int
+svn_config_enumerate_sections2 (svn_config_t *cfg,
+                                svn_config_section_enumerator2_t callback,
+                                void *baton, apr_pool_t *pool)
+{
+  apr_hash_index_t *sec_ndx;
+  apr_pool_t *iteration_pool;
+  int count = 0;
+
+  iteration_pool = svn_pool_create (pool);
+  for (sec_ndx = apr_hash_first (pool, cfg->sections);
+       sec_ndx != NULL;
+       sec_ndx = apr_hash_next (sec_ndx))
+    {
+      void *sec_ptr;
+      cfg_section_t *sec;
+
+      apr_hash_this (sec_ndx, NULL, NULL, &sec_ptr);
+      sec = sec_ptr;
+      ++count;
+      svn_pool_clear (iteration_pool);
+      if (!callback (sec->name, baton, iteration_pool))
+        break;
+    }
+  svn_pool_destroy (iteration_pool);
+
+  return count;
+}
+
+
 
 int
 svn_config_enumerate (svn_config_t *cfg, const char *section,
@@ -600,13 +723,15 @@ svn_config_enumerate (svn_config_t *cfg, const char *section,
   cfg_section_t *sec;
   apr_hash_index_t *opt_ndx;
   int count;
+  apr_pool_t *subpool;
 
   find_option (cfg, section, NULL, &sec);
   if (sec == NULL)
     return 0;
 
+  subpool = svn_pool_create (cfg->x_pool);
   count = 0;
-  for (opt_ndx = apr_hash_first (cfg->x_pool, sec->options);
+  for (opt_ndx = apr_hash_first (subpool, sec->options);
        opt_ndx != NULL;
        opt_ndx = apr_hash_next (opt_ndx))
     {
@@ -623,6 +748,46 @@ svn_config_enumerate (svn_config_t *cfg, const char *section,
         break;
     }
 
+  svn_pool_destroy (subpool);
+  return count;
+}
+
+
+int
+svn_config_enumerate2 (svn_config_t *cfg, const char *section,
+                       svn_config_enumerator2_t callback, void *baton,
+                       apr_pool_t *pool)
+{
+  cfg_section_t *sec;
+  apr_hash_index_t *opt_ndx;
+  apr_pool_t *iteration_pool;
+  int count;
+
+  find_option (cfg, section, NULL, &sec);
+  if (sec == NULL)
+    return 0;
+
+  iteration_pool = svn_pool_create (pool);
+  count = 0;
+  for (opt_ndx = apr_hash_first (pool, sec->options);
+       opt_ndx != NULL;
+       opt_ndx = apr_hash_next (opt_ndx))
+    {
+      void *opt_ptr;
+      cfg_option_t *opt;
+      const char *temp_value;
+
+      apr_hash_this (opt_ndx, NULL, NULL, &opt_ptr);
+      opt = opt_ptr;
+
+      ++count;
+      make_string_from_option (&temp_value, cfg, sec, opt, NULL);
+      svn_pool_clear (iteration_pool);
+      if (!callback (opt->name, temp_value, baton, iteration_pool))
+        break;
+    }
+  svn_pool_destroy (iteration_pool);
+
   return count;
 }
 
@@ -642,12 +807,13 @@ struct search_groups_baton
  */
 static svn_boolean_t search_groups (const char *name,
                                     const char *value,
-                                    void *baton)
+                                    void *baton,
+                                    apr_pool_t *pool)
 {
   struct search_groups_baton *b = baton;
   apr_array_header_t *list;
 
-  list = svn_cstring_split (value, ",", TRUE, b->pool);
+  list = svn_cstring_split (value, ",", TRUE, pool);
   if (svn_cstring_match_glob_list (b->key, list))
     {
       /* Fill in the match and return false, to stop enumerating. */
@@ -668,7 +834,7 @@ const char *svn_config_find_group (svn_config_t *cfg, const char *key,
   gb.key = key;
   gb.match = NULL;
   gb.pool = pool;
-  svn_config_enumerate (cfg, master_section, search_groups, &gb);
+  svn_config_enumerate2 (cfg, master_section, search_groups, &gb, pool);
   return gb.match;
 }
 
@@ -699,20 +865,25 @@ svn_config_get_server_setting_int (svn_config_t *cfg,
 {
   const char* tmp_value;
   char *end_pos;
-  char *default_value_str = apr_psprintf (pool,
-                                          "%" APR_INT64_T_FMT,
-                                          default_value); 
-  tmp_value = svn_config_get_server_setting (cfg, server_group,
-                                             option_name, default_value_str);
 
-  /* read tmp_value as an int now */
-  *result_value = apr_strtoi64 (tmp_value, &end_pos, 0);
-  
-  if (*end_pos != 0) 
+  tmp_value = svn_config_get_server_setting (cfg, server_group,
+                                             option_name, NULL);
+  if (tmp_value == NULL)
+    *result_value = default_value;
+  else
     {
-      return svn_error_create (SVN_ERR_RA_DAV_INVALID_CONFIG_VALUE, NULL,
-                               "non-integer in integer option");
+      /* read tmp_value as an int now */
+      *result_value = apr_strtoi64 (tmp_value, &end_pos, 0);
+
+      if (*end_pos != 0)
+        {
+          return svn_error_createf
+            (SVN_ERR_RA_DAV_INVALID_CONFIG_VALUE, NULL,
+             _("Config error: invalid integer value '%s'"),
+             tmp_value);
+        }
     }
-  return NULL;
+
+  return SVN_NO_ERROR;
 }
 
