@@ -29,6 +29,7 @@
 #include "svn_private_config.h"         /* for SVN_PATH_LOCAL_SEPARATOR */
 #include "svn_utf.h"
 #include "svn_io.h"                     /* for svn_io_stat() */
+#include "svn_ctype.h"
 
 
 /* The canonical empty path.  Can this be changed?  Well, change the empty
@@ -607,7 +608,27 @@ svn_path_is_child (const char *path1,
   return NULL;
 }
 
+svn_boolean_t
+svn_path_is_ancestor (const char *path1, const char *path2)
+{
+  apr_size_t path1_len = strlen (path1);
 
+  /* If path1 is empty and path2 is not absoulte, then path1 is an ancestor. */
+  if (SVN_PATH_IS_EMPTY (path1))
+    return *path2 != '/';
+
+  /* If path1 is a prefix of path2, then:
+     - If path1 ends in a path separator,
+     - If the paths are of the same length
+     OR
+     - path2 starts a new path component after the common prefix,
+     then path1 is an ancestor. */
+  if (strncmp (path1, path2, path1_len) == 0)
+    return path1[path1_len - 1] == '/'
+      || (path2[path1_len] == '/' || path2[path1_len] == '\0');
+
+  return FALSE;
+}
 apr_array_header_t *
 svn_path_decompose (const char *path,
                     apr_pool_t *pool)
@@ -642,7 +663,6 @@ svn_path_decompose (const char *path,
       if ((path[i] == '/') || (path[i] == '\0'))
         {
           if (SVN_PATH_IS_PLATFORM_EMPTY (path + oldi, i - oldi))
-            /* ### Should canonicalization strip "//" and "/./" substrings? */
             *((const char **) apr_array_push (components)) = SVN_EMPTY_PATH;
           else
             *((const char **) apr_array_push (components))
@@ -708,43 +728,17 @@ svn_path_is_backpath_present (const char *path)
    NULL if PATH doesn't appear to be a valid URI.  The returned value
    is not alloced -- it shares memory with PATH. */
 static const char *
-skip_uri_schema (const char *path)
+skip_uri_scheme (const char *path)
 {
   apr_size_t j;
-  apr_size_t len = strlen (path);
 
-  /* ### Taking strlen() initially is inefficient.  It's a holdover
-     from svn_stringbuf_t days. */
+  for (j = 0; path[j]; ++j)
+    if (path[j] == ':' || path[j] == '/')
+       break;
 
-  /* Make sure we have enough characters to even compare. */
-  if (len < 4)
-    return NULL;
+  if (j > 0 && path[j] == ':' && path[j+1] == '/' && path[j+2] == '/')
+    return path + j + 3;
 
-  /* Look for the sequence '://' */
-  for (j = 0; j < len - 3; j++)
-    {
-      /* We hit a '/' before finding the sequence. */
-      if (path[j] == '/')
-        return NULL;
-
-      /* Skip stuff up to the first ':'. */
-      if (path[j] != ':')
-        continue;
-
-      /* Current character is a ':' now.  It better not be the first
-         character. */
-      if (j == 0)
-        return NULL;
-
-      /* Expecting the next two chars to be '/' */
-
-      if ((path[j + 1] == '/')
-          && (path[j + 2] == '/'))
-        return path + j + 3;
-      
-      return NULL;
-    }
-     
   return NULL;
 }
 
@@ -760,7 +754,7 @@ svn_path_is_url (const char *path)
 
      Someday it might be nice to have an actual URI parser here.
   */
-  return skip_uri_schema (path) ? TRUE : FALSE;
+  return skip_uri_scheme (path) ? TRUE : FALSE;
 }
 
 
@@ -778,7 +772,7 @@ svn_path_is_url (const char *path)
 
       alphanum | mark | ":" | "@" | "&" | "=" | "+" | "$" | "," 
 */
-static const int uri_char_validity[256] = {
+static const char uri_char_validity[256] = {
   0, 0, 0, 0, 0, 0, 0, 0,   0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0,   0, 0, 0, 0, 0, 0, 0, 0,
   0, 1, 0, 0, 1, 0, 1, 1,   1, 1, 1, 1, 1, 1, 1, 1,
@@ -808,14 +802,14 @@ svn_path_is_uri_safe (const char *path)
 {
   apr_size_t i;
 
-  /* Skip the schema. */
-  path = skip_uri_schema (path);
+  /* Skip the URI scheme. */
+  path = skip_uri_scheme (path);
 
-  /* No schema?  Get outta here. */
+  /* No scheme?  Get outta here. */
   if (! path)
     return FALSE;
 
-  /* Skip to the first slash that's after the schema. */
+  /* Skip to the first slash that's after the URI scheme. */
   path = strchr (path, '/');
 
   /* If there's no first slash, then there's only a host portion;
@@ -849,7 +843,7 @@ svn_path_is_uri_safe (const char *path)
    If no encoding was needed, return PATH, else return a new string allocated
    in POOL. */
 static const char *
-uri_escape (const char *path, const int table[], apr_pool_t *pool)
+uri_escape (const char *path, const char table[], apr_pool_t *pool)
 {
   svn_stringbuf_t *retstr;
   apr_size_t i, copied = 0;
@@ -914,7 +908,7 @@ svn_path_uri_encode (const char *path, apr_pool_t *pool)
     return ret;
 }
 
-static const int iri_escape_chars[256] = {
+static const char iri_escape_chars[256] = {
   1, 1, 1, 1, 1, 1, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1,
   1, 1, 1, 1, 1, 1, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1,
   1, 1, 1, 1, 1, 1, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1,
@@ -941,7 +935,7 @@ svn_path_uri_from_iri (const char *iri, apr_pool_t *pool)
   return uri_escape (iri, iri_escape_chars, pool);
 }
 
-const int uri_autoescape_chars[256] = {
+const char uri_autoescape_chars[256] = {
   1, 1, 1, 1, 1, 1, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1,
   1, 1, 1, 1, 1, 1, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1,
   0, 1, 0, 1, 1, 1, 1, 1,  1, 1, 1, 1, 1, 1, 1, 1,
@@ -1062,8 +1056,8 @@ svn_path_get_absolute(const char **pabsolute,
 
       if (apr_err)
         return svn_error_createf(SVN_ERR_BAD_FILENAME, NULL,
-                                 "Couldn't determine absolute path of '%s'", 
-                                 relative);
+                                 _("Couldn't determine absolute path of '%s'"), 
+                                 svn_path_local_style (relative, pool));
     }
 
   SVN_ERR (svn_path_cstring_to_utf8 (pabsolute, buffer, pool));
@@ -1101,8 +1095,8 @@ svn_path_split_if_file(const char *path,
   else 
     {
       return svn_error_createf(SVN_ERR_BAD_FILENAME, NULL,
-                               "'%s' is neither a file nor a directory name",
-                               path);
+                               _("'%s' is neither a file nor a directory name"),
+                               svn_path_local_style(path, pool));
     }
 
   return SVN_NO_ERROR;
@@ -1120,8 +1114,8 @@ svn_path_canonicalize (const char *path, apr_pool_t *pool)
 
   dst = canon = apr_pcalloc (pool, strlen (path) + 1);
 
-  /* Copy over the URI shema if present. */
-  src = skip_uri_schema (path);
+  /* Copy over the URI scheme if present. */
+  src = skip_uri_scheme (path);
   if (src)
     {
       uri = TRUE;
@@ -1207,7 +1201,7 @@ get_path_encoding (svn_boolean_t *path_is_utf8, apr_pool_t *pool)
   apr_err = apr_filepath_encoding (&encoding_style, pool);
   if (apr_err)
     return svn_error_wrap_apr (apr_err,
-                               "Can't determine the native path encoding");
+                               _("Can't determine the native path encoding"));
 
   /* ### What to do about APR_FILEPATH_ENCODING_UNKNOWN?
      Well, for now we'll just punt to the svn_utf_ functions;
@@ -1249,3 +1243,23 @@ svn_path_cstring_to_utf8 (const char **path_utf8,
   else
     return svn_utf_cstring_to_utf8 (path_utf8, path_apr, pool);
 }
+
+svn_error_t *
+svn_path_check_valid (const char *path, apr_pool_t *pool)
+{
+  const char *c;
+
+  for (c = path; *c; c++)
+    {
+      if (svn_ctype_iscntrl(*c))
+        {
+          return svn_error_createf (
+            SVN_ERR_FS_PATH_SYNTAX, NULL,
+            _("Invalid control character '0x%02x' in path '%s'"),
+            *c,
+            svn_path_local_style (path, pool));
+        }
+    }
+
+  return SVN_NO_ERROR;
+}
