@@ -35,6 +35,7 @@
 #include "svn_io.h"
 #include "svn_error.h"
 #include "svn_string.h"
+#include "svn_utf.h"
 
 
 struct svn_stream_t {
@@ -129,11 +130,35 @@ svn_stream_printf (svn_stream_t *stream,
   message = apr_pvsprintf (pool, fmt, ap);
   va_end (ap);
   
-  len = strlen(message);
+  len = strlen (message);
   return svn_stream_write (stream, message, &len);
 }
 
 
+svn_error_t *
+svn_stream_printf_from_utf8 (svn_stream_t *stream,
+                             const char *encoding,
+                             apr_pool_t *pool,
+                             const char *fmt,
+                             ...)
+{
+  const char *message, *translated;
+  va_list ap;
+  apr_size_t len;
+
+  va_start (ap, fmt);
+  message = apr_pvsprintf (pool, fmt, ap);
+  va_end (ap);
+
+  SVN_ERR (svn_utf_cstring_from_utf8_ex (&translated, message, encoding,
+                                         NULL, pool));
+  
+  len = strlen (translated);
+
+  return svn_stream_write (stream, translated, &len);
+}
+
+
 svn_error_t *
 svn_stream_readline (svn_stream_t *stream,
                      svn_stringbuf_t **stringbuf,
@@ -183,7 +208,7 @@ svn_stream_readline (svn_stream_t *stream,
 svn_error_t *svn_stream_copy (svn_stream_t *from, svn_stream_t *to,
                               apr_pool_t *pool)
 {
-  char buf[SVN_STREAM_CHUNK_SIZE];
+  char *buf = apr_palloc (pool, SVN_STREAM_CHUNK_SIZE);
   apr_size_t len;
 
   /* Read and write chunks until we get a short read, indicating the
@@ -191,11 +216,11 @@ svn_error_t *svn_stream_copy (svn_stream_t *from, svn_stream_t *to,
      associated error.) */
   while (1)
     {
-      len = sizeof (buf); 
+      len = SVN_STREAM_CHUNK_SIZE;
       SVN_ERR (svn_stream_read (from, buf, &len));
       if (len > 0)
         SVN_ERR (svn_stream_write (to, buf, &len));
-      if (len != sizeof (buf))
+      if (len != SVN_STREAM_CHUNK_SIZE)
         break;
     }
   return SVN_NO_ERROR;
@@ -429,7 +454,7 @@ read_handler_gz (void *baton, char *buffer, apr_size_t *len)
       SVN_ERR (zerr_to_svn_error (zerr, "inflateInit", btn->in));
     }
   
-  btn->in->next_out = buffer;
+  btn->in->next_out = (Bytef *) buffer;
   btn->in->avail_out = *len;
   
   while (btn->in->avail_out > 0) 
@@ -480,7 +505,7 @@ write_handler_gz (void *baton, const char *buffer, apr_size_t *len)
   subpool = svn_pool_create (btn->pool);
   write_buf = apr_palloc (subpool, buf_size);
   
-  btn->out->next_in = (char *) buffer;
+  btn->out->next_in = (Bytef *) buffer;  /* Casting away const! */
   btn->out->avail_in = *len;
   
   while (btn->out->avail_in > 0)
