@@ -26,10 +26,7 @@
 #define APR_WANT_STRFUNC
 #include <apr_want.h>
 
-#include "svn_string.h"
-#include "svn_pools.h"
 #include "svn_error.h"
-#include "svn_props.h"
 #include "svn_private_config.h"
 
 
@@ -39,7 +36,7 @@
 svn_error_t *
 svn_mime_type_validate (const char *mime_type, apr_pool_t *pool)
 {
-  /* Since svn:mine-type can actually contain a full content type
+  /* Since svn:mime-type can actually contain a full content type
      specification, e.g., "text/html; charset=UTF-8", make sure we're
      only looking at the media type here. */
   const apr_size_t len = strcspn (mime_type, "; ");
@@ -70,7 +67,7 @@ svn_mime_type_is_binary (const char *mime_type)
   /* See comment in svn_mime_type_validate() above. */
   const apr_size_t len = strcspn (mime_type, "; ");
   return ((strncmp (mime_type, "text/", 5) != 0)
-          && (strncmp (mime_type, "image/x-xbitmap", len) != 0)
-          && (strncmp (mime_type, "image/x-xpixmap", len) != 0)
+          && (len != 15 || strncmp (mime_type, "image/x-xbitmap", len) != 0)
+          && (len != 15 || strncmp (mime_type, "image/x-xpixmap", len) != 0)
           );
 }
