@@ -22,16 +22,15 @@
 #include <apr_pools.h>
 #include <apr_file_io.h>
 #include <apr_time.h>
-#include <apr_strings.h>
 #include "svn_pools.h"
 #include "svn_types.h"
 #include "svn_string.h"
 #include "svn_error.h"
-#include "svn_hash.h"
 #include "svn_path.h"
 #include "svn_time.h"
 #include "svn_wc.h"
 #include "svn_io.h"
+#include "svn_props.h"
 
 #include "wc.h"
 #include "adm_files.h"
@@ -60,7 +59,8 @@ svn_wc_check_wc (const char *path,
   if (kind == svn_node_none)
     {
       return svn_error_createf
-        (APR_ENOENT, NULL, _("'%s' does not exist"), path);
+        (APR_ENOENT, NULL, _("'%s' does not exist"),
+         svn_path_local_style (path, pool));
     }
   else if (kind != svn_node_dir)
     *wc_format = 0;
@@ -103,7 +103,7 @@ svn_wc__check_format (int wc_format, const char *path, apr_pool_t *pool)
         (SVN_ERR_WC_UNSUPPORTED_FORMAT, NULL,
          _("Working copy format of '%s' is too old (%d); "
            "please check out your working copy again"),
-         path, wc_format);
+         svn_path_local_style (path, pool), wc_format);
     }
   else if (wc_format > SVN_WC__VERSION)
     {
@@ -111,7 +111,7 @@ svn_wc__check_format (int wc_format, const char *path, apr_pool_t *pool)
         (SVN_ERR_WC_UNSUPPORTED_FORMAT, NULL,
          _("This client is too old to work with working copy '%s'; "
            "please get a newer Subversion client"),
-         path);
+         svn_path_local_style (path, pool));
     }
 
   return SVN_NO_ERROR;
@@ -168,7 +168,8 @@ svn_wc__timestamps_equal_p (svn_boolean_t *equal_p,
   if (entry == NULL)
     return svn_error_createf
       (SVN_ERR_ENTRY_NOT_FOUND, NULL,
-       _("'%s' is not under version control"), path);
+       _("'%s' is not under version control"),
+       svn_path_local_style (path, pool));
 
   /* Get the timestamp from the working file and the entry */
   if (timestamp_kind == svn_wc__text_time)
@@ -277,7 +278,7 @@ compare_and_verify (svn_boolean_t *modified_p,
   SVN_ERR (svn_wc_translated_file (&tmp_vfile, versioned_file, adm_access,
                                    TRUE, pool));
 
-  /* Compare the files, while calculating the base file's checksum. */
+  /* Compare the files, while maybe calculating the base file's checksum. */
   {
     /* "v_" means versioned_file, "b_" means base_file. */
     svn_error_t *v_err = SVN_NO_ERROR;
@@ -291,13 +292,13 @@ compare_and_verify (svn_boolean_t *modified_p,
     int identical = TRUE;
     unsigned char digest[APR_MD5_DIGESTSIZE];
     apr_md5_ctx_t context;
-    const char *checksum;
 
     SVN_ERR (svn_io_file_open (&v_file_h, tmp_vfile,
                                APR_READ, APR_OS_DEFAULT, pool));
     SVN_ERR (svn_io_file_open (&b_file_h, base_file, APR_READ, APR_OS_DEFAULT,
                                pool));
-    apr_md5_init (&context);
+    if (entry->checksum)
+      apr_md5_init (&context);
 
     loop_pool = svn_pool_create (pool);
     do
@@ -318,7 +319,8 @@ compare_and_verify (svn_boolean_t *modified_p,
         if (b_err && !APR_STATUS_IS_EOF(b_err->apr_err))
           return b_err;
         
-        apr_md5_update (&context, b_buf, b_file_bytes_read);
+        if (entry->checksum)
+          apr_md5_update (&context, b_buf, b_file_bytes_read);
 
         if ((v_err && (! b_err))
             || (v_file_bytes_read != b_file_bytes_read)
@@ -338,17 +340,22 @@ compare_and_verify (svn_boolean_t *modified_p,
     SVN_ERR (svn_io_file_close (v_file_h, pool));
     SVN_ERR (svn_io_file_close (b_file_h, pool));
 
-    apr_md5_final (digest, &context);
-
-    checksum = svn_md5_digest_to_cstring (digest, pool);
-    if (entry->checksum && strcmp (checksum, entry->checksum) != 0)
+    if (entry->checksum)
       {
-        return svn_error_createf
-          (SVN_ERR_WC_CORRUPT_TEXT_BASE, NULL,
-           _("Checksum mismatch indicates corrupt text base: '%s'\n"
-             "   expected:  %s\n"
-             "     actual:  %s\n"),
-           base_file, entry->checksum, checksum);
+        const char *checksum;
+        apr_md5_final (digest, &context);
+        checksum = svn_md5_digest_to_cstring_display (digest, pool);
+        if (strcmp (checksum, entry->checksum) != 0)
+          {
+            return svn_error_createf
+              (SVN_ERR_WC_CORRUPT_TEXT_BASE, NULL,
+               _("Checksum mismatch indicates corrupt text base: '%s'\n"
+                 "   expected:  %s\n"
+                 "     actual:  %s\n"),
+               svn_path_local_style (base_file, pool),
+               entry->checksum,
+               checksum);
+          }
       }
 
     *modified_p = ! identical;
@@ -379,14 +386,7 @@ svn_wc_text_modified_p (svn_boolean_t *modified_p,
   svn_boolean_t equal_timestamps;
   apr_pool_t *subpool = svn_pool_create (pool);
   svn_node_kind_t kind;
-
-  /* Sanity check:  if the path doesn't exist, return FALSE. */
-  SVN_ERR (svn_io_check_path (filename, &kind, subpool));
-  if (kind != svn_node_file)
-    {
-      *modified_p = FALSE;
-      goto cleanup;
-    }
+  svn_error_t *err;
 
   if (! force_comparison)
     {
@@ -395,16 +395,36 @@ svn_wc_text_modified_p (svn_boolean_t *modified_p,
          theoretically, be wrong in certain rare cases, but with the
          addition of a forced delay after commits (see revision 419
          and issue #542) it's highly unlikely to be a problem. */
-      SVN_ERR (svn_wc__timestamps_equal_p (&equal_timestamps,
-                                           filename, adm_access,
-                                           svn_wc__text_time, subpool));
-      if (equal_timestamps)
+      err = svn_wc__timestamps_equal_p (&equal_timestamps,
+                                        filename, adm_access,
+                                        svn_wc__text_time, subpool);
+
+      /* We only care whether there was an error or not, so make sure it
+         is cleared. */
+      svn_error_clear (err);
+
+      /* If we have an error, we fall back on the slower code path below.
+         It might be tempting to optimize this further, for example by
+         detecting when the file didn't exists.  But we have to be careful
+         with what error codes we return.  If the file doesn't exist,
+         we should return no error.  But, *if* it exists, but it is
+         unversioned, we have to return SVN_ERR_ENTRY_NOT_FOUND. */
+      if (! err && equal_timestamps)
         {
           *modified_p = FALSE;
           goto cleanup;
         }
     }
-      
+
+  /* Make sure the file exists before proceeding. */
+  SVN_ERR (svn_io_check_path (filename, &kind, pool));
+  if (kind != svn_node_file)
+    {
+      /* If the file doesn't exist, consider it non-modified. */
+      *modified_p = FALSE;
+      goto cleanup;
+    }
+
   /* If there's no text-base file, we have to assume the working file
      is modified.  For example, a file scheduled for addition but not
      yet committed. */
