@@ -58,12 +58,14 @@ enum dav_svn_flag {
 
 /* per-dir configuration */
 typedef struct {
-  const char *fs_path;              /* path to the SVN FS */
-  const char *repo_name;            /* repository name */
-  const char *xslt_uri;             /* XSL transform URI */
-  const char *fs_parent_path;       /* path to parent of SVN FS'es  */
-  enum dav_svn_flag autoversioning; /* whether autoversioning is active */
-  enum dav_svn_flag do_path_authz;  /* whether GET subrequests are active */
+  const char *fs_path;               /* path to the SVN FS */
+  const char *repo_name;             /* repository name */
+  const char *xslt_uri;              /* XSL transform URI */
+  const char *fs_parent_path;        /* path to parent of SVN FS'es  */
+  enum dav_svn_flag autoversioning;  /* whether autoversioning is active */
+  enum dav_svn_flag do_path_authz;   /* whether GET subrequests are active */
+  enum dav_svn_flag list_parentpath; /* whether to allow GET of parentpath */
+
 } dav_svn_dir_conf;
 
 #define INHERIT_VALUE(parent, child, field) \
@@ -136,6 +138,7 @@ static void *dav_svn_merge_dir_config(apr_pool_t *p,
     newconf->fs_parent_path = INHERIT_VALUE(parent, child, fs_parent_path);
     newconf->autoversioning = INHERIT_VALUE(parent, child, autoversioning);
     newconf->do_path_authz = INHERIT_VALUE(parent, child, do_path_authz);
+    newconf->list_parentpath = INHERIT_VALUE(parent, child, list_parentpath);
 
     return newconf;
 }
@@ -187,6 +190,20 @@ static const char *dav_svn_pathauthz_cmd(cmd_parms *cmd, void *config,
 }
 
 
+static const char *dav_svn_list_parentpath_cmd(cmd_parms *cmd, void *config,
+                                               int arg)
+{
+  dav_svn_dir_conf *conf = config;
+
+  if (arg)
+    conf->list_parentpath = DAV_SVN_FLAG_ON;
+  else
+    conf->list_parentpath = DAV_SVN_FLAG_OFF;
+
+  return NULL;
+}
+
+
 static const char *dav_svn_path_cmd(cmd_parms *cmd, void *config,
                                     const char *arg1)
 {
@@ -350,6 +367,14 @@ svn_boolean_t dav_svn_get_pathauthz_flag(request_rec *r)
     return conf->do_path_authz != DAV_SVN_FLAG_OFF;
 }
 
+svn_boolean_t dav_svn_get_list_parentpath_flag(request_rec *r)
+{
+    dav_svn_dir_conf *conf;
+
+    conf = ap_get_module_config(r->per_dir_config, &dav_svn_module);
+    return conf->list_parentpath == DAV_SVN_FLAG_ON;
+}
+
 static void merge_xml_filter_insert(request_rec *r)
 {
     /* We only care about MERGE and DELETE requests. */
@@ -498,6 +523,10 @@ static const command_rec dav_svn_cmds[] =
                ACCESS_CONF|RSRC_CONF,
                "control path-based authz by enabling/disabling subrequests"),
 
+  /* per directory/location */
+  AP_INIT_FLAG("SVNListParentPath", dav_svn_list_parentpath_cmd, NULL,
+               ACCESS_CONF|RSRC_CONF, "allow GET of SVNParentPath."),
+
   { NULL }
 };
 
