@@ -56,13 +56,13 @@ def externals_test_setup(sbox):
 
   The arrangement of the externals in the first repository is:
 
-     /A/B/     ==>  exdir_G       <schema>:///<other_repos>/A/D/G
-                    exdir_H  -r 1 <schema>:///<other_repos>/A/D/H
+     /A/C/     ==>  exdir_G       <scheme>:///<other_repos>/A/D/G
+                    exdir_H  -r 1 <scheme>:///<other_repos>/A/D/H
 
-     /A/D/     ==>  exdir_A          <schema>:///<other_repos>/A
-                    exdir_A/G        <schema>:///<other_repos>/A/D/G
-                    exdir_A/H  -r 3  <schema>:///<other_repos>/A/D/H
-                    x/y/z/blah       <schema>:///<other_repos>/A/B/E
+     /A/D/     ==>  exdir_A          <scheme>:///<other_repos>/A
+                    exdir_A/G        <scheme>:///<other_repos>/A/D/G
+                    exdir_A/H  -r 3  <scheme>:///<other_repos>/A/D/H
+                    x/y/z/blah       <scheme>:///<other_repos>/A/B
 
   NOTE: Before calling this, use externals_test_cleanup(SBOX) to
   remove a previous incarnation of the other repository.
@@ -85,7 +85,7 @@ def externals_test_setup(sbox):
 
   # These are the directories on which `svn:externals' will be set, in
   # revision 6 on the first repo.
-  B_path = os.path.join(wc_init_dir, "A/B")
+  C_path = os.path.join(wc_init_dir, "A/C")
   D_path = os.path.join(wc_init_dir, "A/D")
 
   # Create a working copy.
@@ -98,22 +98,22 @@ def externals_test_setup(sbox):
   # Make revisions 2 through 5, but don't bother with pre- and
   # post-commit status checks.
 
-  svntest.main.file_append(mu_path, "\nAdded to mu in revision 2.\n")
+  svntest.main.file_append(mu_path, "Added to mu in revision 2.\n")
   svntest.actions.run_and_verify_svn("", None, [],
                                      'ci', '-m', 'log msg',
                                      '--quiet', wc_init_dir)
 
-  svntest.main.file_append(pi_path, "\nAdded to pi in revision 3.\n")
+  svntest.main.file_append(pi_path, "Added to pi in revision 3.\n")
   svntest.actions.run_and_verify_svn("", None, [],
                                      'ci', '-m', 'log msg',
                                      '--quiet', wc_init_dir)
 
-  svntest.main.file_append(lambda_path, "\nAdded to lambda in revision 4.\n")
+  svntest.main.file_append(lambda_path, "Added to lambda in revision 4.\n")
   svntest.actions.run_and_verify_svn("", None, [],
                                      'ci', '-m', 'log msg',
                                      '--quiet', wc_init_dir)
 
-  svntest.main.file_append(omega_path, "\nAdded to omega in revision 5.\n")
+  svntest.main.file_append(omega_path, "Added to omega in revision 5.\n")
   svntest.actions.run_and_verify_svn("", None, [],
                                      'ci', '-m', 'log msg',
                                      '--quiet', wc_init_dir)
@@ -136,7 +136,7 @@ def externals_test_setup(sbox):
   svntest.main.file_append(tmp_f, externals_desc)
   svntest.actions.run_and_verify_svn("", None, [],
                                      'pset',
-                                     '-F', tmp_f, 'svn:externals', B_path)
+                                     '-F', tmp_f, 'svn:externals', C_path)
    
   os.remove(tmp_f)
 
@@ -147,7 +147,7 @@ def externals_test_setup(sbox):
            "\n"                                              + \
            "exdir_A/H   -r 1  " + other_repo_url + "/A/D/H"  + \
            "\n"                                              + \
-           "x/y/z/blah        " + other_repo_url + "/A/B/E"  + \
+           "x/y/z/blah        " + other_repo_url + "/A/B"    + \
            "\n"
 
   svntest.main.file_append(tmp_f, externals_desc)
@@ -159,13 +159,12 @@ def externals_test_setup(sbox):
   # Commit the property changes.
 
   expected_output = svntest.wc.State(wc_init_dir, {
-    'A/B' : Item(verb='Sending'),
+    'A/C' : Item(verb='Sending'),
     'A/D' : Item(verb='Sending'),
     })
 
   expected_status = svntest.actions.get_virginal_state(wc_init_dir, 5)
-  expected_status.tweak(repos_rev=6)
-  expected_status.tweak('A/B', 'A/D', wc_rev=6, status='  ')
+  expected_status.tweak('A/C', 'A/D', wc_rev=6, status='  ')
 
   svntest.actions.run_and_verify_commit(wc_init_dir,
                                         expected_output,
@@ -217,16 +216,16 @@ def checkout_with_externals(sbox):
                                      repo_url, wc_dir)
 
   # Probe the working copy a bit, see if it's as expected.
-  exdir_G_path    = os.path.join(wc_dir, "A/B/exdir_G")
+  exdir_G_path    = os.path.join(wc_dir, "A", "C", "exdir_G")
   exdir_G_pi_path = os.path.join(exdir_G_path, "pi")
-  exdir_H_path       = os.path.join(wc_dir, "A/B/exdir_H")
+  exdir_H_path       = os.path.join(wc_dir, "A", "C", "exdir_H")
   exdir_H_omega_path = os.path.join(exdir_H_path, "omega")
-  x_path     = os.path.join(wc_dir, "A/D/x")
+  x_path     = os.path.join(wc_dir, "A", "D", "x")
   y_path     = os.path.join(x_path, "y")
   z_path     = os.path.join(y_path, "z")
   blah_path  = os.path.join(z_path, "blah")
-  alpha_path = os.path.join(blah_path, "alpha")
-  beta_path  = os.path.join(blah_path, "beta")
+  alpha_path = os.path.join(blah_path, "E", "alpha")
+  beta_path  = os.path.join(blah_path, "E", "beta")
 
   if (not os.path.exists(exdir_G_path)):
     raise svntest.Failure("Probing for " + exdir_G_path + " failed.")
@@ -252,7 +251,7 @@ def checkout_with_externals(sbox):
   # Pick a file at random, make sure it has the expected contents.
   fp = open(exdir_H_omega_path, 'r')
   lines = fp.readlines()
-  if not ((len(lines) == 1) and (lines[0] == "This is the file 'omega'.")):
+  if not ((len(lines) == 1) and (lines[0] == "This is the file 'omega'.\n")):
     raise svntest.Failure("Unexpected contents for rev 1 of " +
                           exdir_H_omega_path)
 
@@ -472,10 +471,10 @@ def update_change_modified_external(sbox):
   # Make a couple of mods in the "x/y/z/blah/" external.
   alpha_path = os.path.join(other_wc_dir, "A", "D",
                             "x", "y", "z", "blah", "alpha")
-  svntest.main.file_append(alpha_path, "\nSome new text in alpha.")
+  svntest.main.file_append(alpha_path, "Some new text in alpha.\n")
   new_file = os.path.join(other_wc_dir, "A", "D",
                           "x", "y", "z", "blah", "fish.txt")
-  svntest.main.file_append(new_file, "This is an unversioned file.")
+  svntest.main.file_append(new_file, "This is an unversioned file.\n")
 
   # Change the "x/y/z/blah" external on A/D to point to a different
   # URL.  There are some local mods under the old checked-out external,
@@ -532,13 +531,12 @@ def update_receive_change_under_external(sbox):
 
   # Commit some modifications from the other_wc.
   other_gamma_path = os.path.join(other_wc_dir, 'A', 'D', 'gamma')
-  svntest.main.file_append(other_gamma_path, "\nNew text in other gamma.")
+  svntest.main.file_append(other_gamma_path, "New text in other gamma.\n")
 
   expected_output = svntest.wc.State(other_wc_dir, {
     'A/D/gamma' : Item(verb='Sending'),
     })
   expected_status = svntest.actions.get_virginal_state(other_wc_dir, 5)
-  expected_status.tweak(repos_rev=6)
   expected_status.tweak('A/D/gamma', wc_rev=6)
   svntest.actions.run_and_verify_commit(other_wc_dir,
                                         expected_output,
@@ -561,20 +559,19 @@ def update_receive_change_under_external(sbox):
   lines = fp.readlines()
   if not ((len(lines) == 2)
           and (lines[0] == "This is the file 'gamma'.\n")
-          and (lines[1] == "New text in other gamma.")):
+          and (lines[1] == "New text in other gamma.\n")):
     raise svntest.Failure("Unexpected contents for externally modified " +
                           external_gamma_path)
   fp.close()
 
   # Commit more modifications
   other_rho_path = os.path.join(other_wc_dir, 'A', 'D', 'G', 'rho')
-  svntest.main.file_append(other_rho_path, "\nNew text in other rho.")
+  svntest.main.file_append(other_rho_path, "New text in other rho.\n")
 
   expected_output = svntest.wc.State(other_wc_dir, {
     'A/D/G/rho' : Item(verb='Sending'),
     })
   expected_status = svntest.actions.get_virginal_state(other_wc_dir, 5)
-  expected_status.tweak(repos_rev=7)
   expected_status.tweak('A/D/gamma', wc_rev=6)
   expected_status.tweak('A/D/G/rho', wc_rev=7)
   svntest.actions.run_and_verify_commit(other_wc_dir,
@@ -584,14 +581,14 @@ def update_receive_change_under_external(sbox):
                                         other_wc_dir)
 
   svntest.actions.run_and_verify_svn("", None, [],
-                                     'up', os.path.join(wc_dir, "A", "B"))
+                                     'up', os.path.join(wc_dir, "A", "C"))
 
-  external_rho_path = os.path.join(wc_dir, 'A', 'B', 'exdir_G', 'rho')
+  external_rho_path = os.path.join(wc_dir, 'A', 'C', 'exdir_G', 'rho')
   fp = open(external_rho_path, 'r')
   lines = fp.readlines()
   if not ((len(lines) == 2)
           and (lines[0] == "This is the file 'rho'.\n")
-          and (lines[1] == "New text in other rho.")):
+          and (lines[1] == "New text in other rho.\n")):
     raise svntest.Failure("Unexpected contents for externally modified " +
                           external_rho_path)
   fp.close()
@@ -667,17 +664,24 @@ def disallow_dot_or_dotdot_directory_reference(sbox):
   H_path = os.path.join(wc_dir, 'A', 'D', 'H')
   C_path = os.path.join(wc_dir, 'A', 'C')
   F_path = os.path.join(wc_dir, 'A', 'C', 'F')
-  externals_value_1 = "../foo  "         + repo_url + "/A/B/E" + "\n"
-  externals_value_2 = "foo/bar/../baz  " + repo_url + "/A/B/E" + "\n"
-  externals_value_3 = "foo/..  "         + repo_url + "/A/B/E" + "\n"
-  externals_value_4 = ". "               + repo_url + "/A/B/E" + "\n"
-  externals_value_5 = "./ "              + repo_url + "/A/B/E" + "\n"
+  externals_value_1 = "../foo"         + " " + repo_url + "/A/B/E" + "\n"
+  externals_value_2 = "foo/bar/../baz" + " " + repo_url + "/A/B/E" + "\n"
+  externals_value_3 = "foo/.."         + " " + repo_url + "/A/B/E" + "\n"
+  externals_value_4 = "."              + " " + repo_url + "/A/B/E" + "\n"
+  externals_value_5 = "./"             + " " + repo_url + "/A/B/E" + "\n"
+  externals_value_6 = ".."             + " " + repo_url + "/A/B/E" + "\n"
+  externals_value_7 = "././/.///."     + " " + repo_url + "/A/B/E" + "\n"
+  externals_value_8 = "/foo"           + " " + repo_url + "/A/B/E" + "\n"
+
 
   set_externals_for_path_expect_error(B_path, externals_value_1, wc_dir)
   set_externals_for_path_expect_error(G_path, externals_value_2, wc_dir)
   set_externals_for_path_expect_error(H_path, externals_value_3, wc_dir)
   set_externals_for_path_expect_error(C_path, externals_value_4, wc_dir)
   set_externals_for_path_expect_error(F_path, externals_value_5, wc_dir)
+  set_externals_for_path_expect_error(B_path, externals_value_6, wc_dir)
+  set_externals_for_path_expect_error(G_path, externals_value_7, wc_dir)
+  set_externals_for_path_expect_error(H_path, externals_value_8, wc_dir)
 
 
 #----------------------------------------------------------------------
@@ -699,16 +703,16 @@ def export_with_externals(sbox):
                                      repo_url, wc_dir)
 
   # Probe the working copy a bit, see if it's as expected.
-  exdir_G_path    = os.path.join(wc_dir, "A/B/exdir_G")
+  exdir_G_path    = os.path.join(wc_dir, "A", "C", "exdir_G")
   exdir_G_pi_path = os.path.join(exdir_G_path, "pi")
-  exdir_H_path       = os.path.join(wc_dir, "A/B/exdir_H")
+  exdir_H_path       = os.path.join(wc_dir, "A", "C", "exdir_H")
   exdir_H_omega_path = os.path.join(exdir_H_path, "omega")
-  x_path     = os.path.join(wc_dir, "A/D/x")
+  x_path     = os.path.join(wc_dir, "A", "D", "x")
   y_path     = os.path.join(x_path, "y")
   z_path     = os.path.join(y_path, "z")
   blah_path  = os.path.join(z_path, "blah")
-  alpha_path = os.path.join(blah_path, "alpha")
-  beta_path  = os.path.join(blah_path, "beta")
+  alpha_path = os.path.join(blah_path, "E", "alpha")
+  beta_path  = os.path.join(blah_path, "E", "beta")
 
   if (not os.path.exists(exdir_G_path)):
     raise svntest.Failure("Probing for " + exdir_G_path + " failed.")
@@ -741,7 +745,7 @@ def export_with_externals(sbox):
                           exdir_G_pi_path)
   fp = open(exdir_H_omega_path, 'r')
   lines = fp.readlines()
-  if not ((len(lines) == 1) and (lines[0] == "This is the file 'omega'.")):
+  if not ((len(lines) == 1) and (lines[0] == "This is the file 'omega'.\n")):
     raise svntest.Failure("Unexpected contents for rev 1 of " +
                           exdir_H_omega_path)
 
