@@ -57,7 +57,8 @@ del_lines_res = [
 
                  # Also for 'svn --version':
                  re.compile(r"\* ra_(dav|local|svn) :"),
-                 re.compile(r"  - handles '(https?|file|svn)' schema"),
+                 re.compile(r"  - handles '(https?|file|svn)' scheme"),
+                 re.compile(r"\* fs_(base|fs) :"),
                 ]
 
 # This is a list of lines to search and replace text on.
@@ -73,6 +74,13 @@ rep_lines_res = [
                  # In 'svn --version --quiet', we print only the version
                  # number in a single line.
                  (re.compile(r'^\d+\.\d+\.\d+(-[a-zA-Z0-9]+)?$'), 'X.Y.Z\n'),
+                 # 'svn --help' has a line with the version number.
+                 # It can vary, for example:
+                 # "Subversion command-line client, version 1.1.0."
+                 # "Subversion command-line client, version 1.1.0-dev."
+                 (re.compile(r'Subversion command-line client, '
+                             'version \d+\.\d+\.\d+(.|-[a-zA-Z0-9]+\.)$'),
+                  'Subversion command-line client, version X.Y.Z.'),
                 ]
 
 def process_lines(lines):
