@@ -81,6 +81,9 @@ class SVNRepositoryCopyFailure(Failure):
   "Exception raised if unable to copy a repository"
   pass
 
+class SVNRepositoryCreateFailure(Failure):
+  "Exception raised if unable to create a repository"
+  pass
 
 # Windows specifics
 if sys.platform == 'win32':
@@ -121,10 +124,14 @@ if windows == 1:
 # Global variable indicating the FS type for repository creations.
 fs_type = None
 
+# All temporary repositories and working copies are created underneath
+# this dir, so there's one point at which to mount, e.g., a ramdisk.
+work_dir = "svn-test-work"
+
 # Where we want all the repositories and working copies to live.
 # Each test will have its own!
-general_repo_dir = "repositories"
-general_wc_dir = "working_copies"
+general_repo_dir = os.path.join(work_dir, "repositories")
+general_wc_dir = os.path.join(work_dir, "working_copies")
 
 # A relative path that will always point to latest repository
 current_repo_dir = None
@@ -133,7 +140,7 @@ current_repo_url = None
 # temp directory in which we will create our 'pristine' local
 # repository and other scratch data.  This should be removed when we
 # quit and when we startup.
-temp_dir = 'local_tmp'
+temp_dir = os.path.join(work_dir, 'local_tmp')
 
 # (derivatives of the tmp dir.)
 pristine_dir = os.path.join(temp_dir, "repos")
@@ -151,26 +158,26 @@ default_config_dir = config_dir
 #
 _item = wc.StateItem
 greek_state = wc.State('', {
-  'iota'        : _item("This is the file 'iota'."),
+  'iota'        : _item("This is the file 'iota'.\n"),
   'A'           : _item(),
-  'A/mu'        : _item("This is the file 'mu'."),
+  'A/mu'        : _item("This is the file 'mu'.\n"),
   'A/B'         : _item(),
-  'A/B/lambda'  : _item("This is the file 'lambda'."),
+  'A/B/lambda'  : _item("This is the file 'lambda'.\n"),
   'A/B/E'       : _item(),
-  'A/B/E/alpha' : _item("This is the file 'alpha'."),
-  'A/B/E/beta'  : _item("This is the file 'beta'."),
+  'A/B/E/alpha' : _item("This is the file 'alpha'.\n"),
+  'A/B/E/beta'  : _item("This is the file 'beta'.\n"),
   'A/B/F'       : _item(),
   'A/C'         : _item(),
   'A/D'         : _item(),
-  'A/D/gamma'   : _item("This is the file 'gamma'."),
+  'A/D/gamma'   : _item("This is the file 'gamma'.\n"),
   'A/D/G'       : _item(),
-  'A/D/G/pi'    : _item("This is the file 'pi'."),
-  'A/D/G/rho'   : _item("This is the file 'rho'."),
-  'A/D/G/tau'   : _item("This is the file 'tau'."),
+  'A/D/G/pi'    : _item("This is the file 'pi'.\n"),
+  'A/D/G/rho'   : _item("This is the file 'rho'.\n"),
+  'A/D/G/tau'   : _item("This is the file 'tau'.\n"),
   'A/D/H'       : _item(),
-  'A/D/H/chi'   : _item("This is the file 'chi'."),
-  'A/D/H/psi'   : _item("This is the file 'psi'."),
-  'A/D/H/omega' : _item("This is the file 'omega'."),
+  'A/D/H/chi'   : _item("This is the file 'chi'.\n"),
+  'A/D/H/psi'   : _item("This is the file 'psi'.\n"),
+  'A/D/H/omega' : _item("This is the file 'omega'.\n"),
   })
 
 
@@ -180,9 +187,11 @@ greek_state = wc.State('', {
 def get_admin_name():
   "Return name of SVN administrative subdirectory."
 
-  # todo: One day this sucker will try to intelligently discern what
-  # the admin dir is.  For now, '.svn' will suffice.
-  return '.svn'
+  if (windows or sys.platform == 'cygwin') \
+      and os.environ.has_key('SVN_ASP_DOT_NET_HACK'):
+    return '_svn'
+  else:
+    return '.svn'
 
 def get_start_commit_hook_path(repo_dir):
   "Return the path of the start-commit-hook conf file in REPO_DIR."
@@ -202,6 +211,16 @@ def get_post_commit_hook_path(repo_dir):
   return os.path.join(repo_dir, "hooks", "post-commit")
 
 
+def get_authz_file_path(repo_dir):
+  "Return the path of the authz file in REPO_DIR."
+
+  return os.path.join(repo_dir, "conf", "authz")
+
+def get_svnserve_conf_file_path(repo_dir):
+  "Return the path of the svnserve.conf file in REPO_DIR."
+
+  return os.path.join(repo_dir, "conf", "svnserve.conf")
+
 # Run any binary, logging the command line (TODO: and return code)
 def run_command(command, error_expected, binary_mode=0, *varargs):
   """Run COMMAND with VARARGS; return stdout, stderr as lists of lines.
@@ -272,6 +291,28 @@ def reset_config_dir():
 
   config_dir = default_config_dir
 
+def create_config_dir(cfgdir,
+                      config_contents = '#\n',
+                      server_contents = '#\n'):
+  "Create config directories and files"
+
+  # config file names
+  cfgfile_cfg = os.path.join(cfgdir, 'config')
+  cfgfile_srv = os.path.join(cfgdir, 'server')
+
+  # create the directory
+  if not os.path.isdir(cfgdir):
+    os.makedirs(cfgdir)
+
+  fd = open(cfgfile_cfg, 'w')
+  fd.write(config_contents)
+  fd.close()
+
+  fd = open(cfgfile_srv, 'w')
+  fd.write(server_contents)
+  fd.close()
+
+
 # For running subversion and returning the output
 def run_svn(error_expected, *varargs):
   """Run svn with VARARGS; return stdout, stderr as lists of lines.
@@ -351,9 +392,13 @@ def create_repos(path):
   stdout, stderr = run_command(svnadmin_binary, 1, 0, "create", path, *opts)
 
   # Skip tests if we can't create the repository.
-  for line in stderr:
-    if line.find('Unknown FS type') != -1:
-      raise Skip
+  if stderr:
+    for line in stderr:
+      if line.find('Unknown FS type') != -1:
+        raise Skip
+    # If the FS type is known, assume the repos couldn't be created
+    # (e.g. due to a missing 'svnadmin' binary).
+    raise SVNRepositoryCreateFailure("".join(stderr).rstrip())
 
   # Allow unauthenticated users to write to the repos, for ra_svn testing.
   file_append(os.path.join(path, "conf", "svnserve.conf"),
@@ -450,10 +495,18 @@ def canonize_url(input):
 # Sandbox handling
 
 class Sandbox:
-  "Manages a sandbox for a test to operate within."
+  """Manages a sandbox (one or more repository/working copy pairs) for
+  a test to operate within."""
+
+  dependents = None
 
   def __init__(self, module, idx):
-    self.name = '%s-%d' % (module, idx)
+    self._set_name("%s-%d" % (module, idx))
+
+  def _set_name(self, name):
+    """A convenience method for renaming a sandbox, useful when
+    working with multiple repositories in the same unit test."""
+    self.name = name
     self.wc_dir = os.path.join(general_wc_dir, self.name)
     self.repo_dir = os.path.join(general_repo_dir, self.name)
     self.repo_url = test_area_url + '/' + self.repo_dir
@@ -461,6 +514,17 @@ class Sandbox:
       self.repo_url = string.replace(self.repo_url, '\\', '/')
     self.test_paths = [self.wc_dir, self.repo_dir]
 
+  def clone_dependent(self):
+    """A convenience method for creating a near-duplicate of this
+    sandbox, useful when working with multiple repositories in the
+    same unit test.  Any necessary cleanup operations are triggered
+    by cleanup of the original sandbox."""
+    if not self.dependents:
+      self.dependents = []
+    self.dependents.append(copy.deepcopy(self))
+    self.dependents[-1]._set_name("%s-%d" % (self.name, len(self.dependents)))
+    return self.dependents[-1]
+
   def build(self):
     if actions.make_repo_and_wc(self):
       raise Failure("Could not build repository and sandbox '%s'" % self.name)
@@ -482,6 +546,11 @@ class Sandbox:
     return path
 
   def cleanup_test_paths(self):
+    "Clean up detritus from this sandbox, and any dependents."
+    if self.dependents:
+      # Recursively cleanup any dependent sandboxes.
+      for sbox in self.dependents:
+        sbox.cleanup_test_paths()
     for path in self.test_paths:
       _cleanup_test_path(path)
 
@@ -594,7 +663,7 @@ def run_tests(test_list):
     opts, args = my_getopt(sys.argv[1:], 'v',
                            ['url=', 'fs-type=', 'verbose', 'cleanup'])
   except getopt.GetoptError:
-    args = []
+    opts, args = [], []
 
   for arg in args:
     if arg == "list":
@@ -629,6 +698,9 @@ def run_tests(test_list):
     elif opt == "--cleanup":
       cleanup_mode = 1
 
+  if test_area_url[-1:] == '/': # Normalize url to have no trailing slash
+    test_area_url = test_area_url[:-1]
+
   exit_code = _internal_run_tests(test_list, testnum)
 
   # remove all scratchwork: the 'pristine' repository, greek tree, etc.
